
#import "MyTableView.h"

// RGB values for stripe color (light blue)
#define STRIPE_RED   (237.0 / 255.0)
#define STRIPE_GREEN (243.0 / 255.0)
#define STRIPE_BLUE  (254.0 / 255.0)
static NSColor *sStripeColor = nil;

@implementation MyTableView

- (void)awakeFromNib
{
    // by default, we're not playing any songs.
    isRunning=NO;
}

- (IBAction)showWindow:(id)sender
{
  [[myController songSelected] showWindow:self];
}

/*
// When the user double-clicks a song or clicks the play button, this action method is called
- (IBAction)playSong:(id)sender
{
    NSString *pathToPlayer;
    
    // Stop the previous song before starting a new one
    if (isRunning)
        [self stopPlaying:nil];
    
    // We construct a path to the mpg123 command-line binary that is inside the application package
    pathToPlayer=[NSString stringWithFormat:@"%@%@",[[NSBundle mainBundle] privateFrameworksPath],@"/mpg123"];
    
    // If mpg123 doesn't exist inside the application package, then we have a problem; let the user know
    if (![[NSFileManager defaultManager] fileExistsAtPath:pathToPlayer])
    {
        NSRunAlertPanel(@"Error",@"mpg123 (which does the actual music playing) does not exist inside the Frameworks folder of the application package.  Please rebuild and copy in mpg123 (see the ReadMe file for more details).",@"OK",nil,nil);
	return;
    }
    
    // Ok, allocate and initialize a new NSTask
    musicTask=[[NSTask alloc] init];
    
    // Tell the NSTask what the path is to the binary it should launch
    [musicTask setLaunchPath:pathToPlayer];
    
    // The argument that we pass to mpg123 (in the form of an array) is the path to the song
    [musicTask setArguments:[NSArray arrayWithObject:[myController songSelected]]];
    
    // mpg123 has a bug where if the TERM environment variable isn't set, it crashes.
    // NSTask doesn't set this environment variable, so we have to give it a dummy value.
    [musicTask setEnvironment:[NSDictionary dictionaryWithObject:@"dumb" forKey:@"TERM"]];
    
    // Launch the process asynchronously
    [musicTask launch];
    
    // Yep, we're playing a song now!
    isRunning=YES;
}

// It's time to stop playing a song now
-(IBAction)stopPlaying:(id)dummy
{
    if (isRunning)
    {
        // Stop the process that this NSTask kicked off
        [musicTask terminate];
        
        // Release the memory, etc. for this NSTask
        [musicTask release];
        
        isRunning=NO;
    }
}
*/
// This is called after the table background is filled in, but before the cell contents are drawn.
// We override it so we can do our own light-blue row stripes a la iTunes.
- (void) highlightSelectionInClipRect:(NSRect)rect {
    [self drawStripesInRect:rect];
    [super highlightSelectionInClipRect:rect];
}

// This routine does the actual blue stripe drawing, filling in every other row of the table
// with a blue background so you can follow the rows easier with your eyes.
- (void) drawStripesInRect:(NSRect)clipRect {
    NSRect stripeRect;
    float fullRowHeight = [self rowHeight] + [self intercellSpacing].height;
    float clipBottom = NSMaxY(clipRect);
    int firstStripe = clipRect.origin.y / fullRowHeight;
    if (firstStripe % 2 == 0)
        firstStripe++;			// we're only interested in drawing the stripes
                         // set up first rect
    stripeRect.origin.x = clipRect.origin.x;
    stripeRect.origin.y = firstStripe * fullRowHeight;
    stripeRect.size.width = clipRect.size.width;
    stripeRect.size.height = fullRowHeight;
    // set the color
    if (sStripeColor == nil)
        sStripeColor = [[NSColor colorWithCalibratedRed:STRIPE_RED green:STRIPE_GREEN blue:STRIPE_BLUE alpha:1.0] retain];
    [sStripeColor set];
    // and draw the stripes
    while (stripeRect.origin.y < clipBottom) {
        NSRectFill(stripeRect);
        stripeRect.origin.y += fullRowHeight * 2.0;
    }
}



@end
