/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/**
   \class AiffFileIO
   \brief SoundFileIO for .aiff soundfiles
   
   SoundFileIO reader/writer for .aiff sound files.
   (SGI, MAC aiff audio format). 
*/




#if !defined(__AiffFileIO_h)
#define __AiffFileIO_h





#include "MSignal.h"

#include <stdio.h>
#include "defs.h"
#include "SoundFileIO.h"

#include <string.h>
#include <iostream.h>
using namespace std;



typedef struct
{
  char chunkID[4];			/* 'FORM' */
  long chunkDataSize;			
  char formType[4];
} FormAIFCChunk_Header;

typedef struct
{
  char chunkID[4];				/* chunk ID */
  long chunkDataSize;			        /* chunk data size, in bytes */
} Chunk_Header;


typedef struct
{
  char chunkID[4];
  long chunk_DataSize;
  short channels;
  unsigned long length;
  short sample_size;
  double sample_rate;
} CommonChunk_Header;





class AiffFileIO: public SoundFileIO
{
private:
  CommonChunk_Header hdr_;
  

public:
  AiffFileIO(string filename, int mode);
  ~AiffFileIO();
  void init();
  int check_type();
  void get(fvec &win, unsigned long index);
  void put(fvec &win, unsigned long index);
  

};




#endif

	

	
