/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class AuFileIO
   \brief SoundFileIO for .au (.snd) soundfiles
   
   SoundFileIO reader/writer for .au (.snd) sound files.
   (Next, Sun audio format). 
*/



/* AuFileIO:
   Soundfile reader/writer for .snd (.au) sound file format. 
*/


#if !defined(__AuFileIO_h)
#define __AuFileIO_h






#include <stdio.h>
#include "MSignal.h"
#include "defs.h"
#include "SoundFileIO.h"

#include <string.h>
using namespace std;

#define SND_MAGIC_NUM 0x2e736e64

/********  NeXT/Sun Soundfile Header Struct   *******/

struct snd_header 
{
  char pref[4];
  long hdrLength;
  long fileLength;
  long mode;
  long srate;
  long channels;
  char comment[1024];
};


/* Array containing descriptions of
the various formats for the samples
of the Next .snd/ Sun .au format */





/* types of .snd files */  
#define SND_FORMAT_UNSPECIFIED 0
#define SND_FORMAT_MULAW_8     1
#define SND_FORMAT_LINEAR_8    2
#define SND_FORMAT_LINEAR_16   3
#define SND_FORMAT_LINEAR_24   4
#define SND_FORMAT_LINEAR_32   5
#define SND_FORMAT_FLOAT       6





      


class AuFileIO: public SoundFileIO
{
private:
  snd_header hdr_;
  long pos_;
  long written_;
  
public:
  AuFileIO(string filename, int mode);
  ~AuFileIO();
  void init();
  int check_type();
  void get(fvec &win, unsigned long index);
  void put(fvec &win, unsigned long index);
  float *getall();
};




#endif

	
