/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class AutoCorrelationFFT
    \brief AutoCorrelation System based on FFT


*/



#include "AutoCorrelationFFT.h"

AutoCorrelationFFT::AutoCorrelationFFT()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = DEFAULT_WIN_SIZE;
  temp_.create(inSize_*2);
}

AutoCorrelationFFT::AutoCorrelationFFT(unsigned int inSize)
{
  inSize_ = inSize;
  outSize_ = inSize;
  temp_.create(inSize_*2);
}

float 
AutoCorrelationFFT::pitch()
{
  return pitch_;
}

AutoCorrelationFFT::~AutoCorrelationFFT()
{
}


void 
AutoCorrelationFFT::process(fvec& in, fvec& out) 
{
  
  // plotter_.plot_wait(in);
  
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  unsigned int i;

  for(i=0; i<inSize_; i++)
  {
    temp_(i) = in(i);
    temp_(i+inSize_) = 0;
  }
  float *temp = temp_.getData();
  fft_.rfft(temp, inSize_, FFT_FORWARD);
  temp[0] = 0.0;
  temp[1] = 0.0;
  
  /* Computer magnitude */
  for (i=1; i<inSize_; i++)
    {
      temp[2*i] = sqrt(temp[2*i]*temp[2*i] + temp[2*i+1]*temp[2*i+1]);
      temp[2*i+1] = 0.0;
    }  
  fft_.rfft(temp, inSize_, FFT_INVERSE);
  for (i=0; i < outSize_; i++) out(i) = temp[i]; 
}
	
