#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "FeatMatrix.h"





using namespace std;
vector<string> featNames_;
vector<string> labelNames_;

struct ltstr
{
  bool operator()(const char* s1, const char* s2) const
  {
    return strcmp(s1, s2) < 0;
  }
};


Collection 
readCollection(string name)
{
  string collectionstr = MARSYAS_MFDIR;
  collectionstr += "/collections/";
  collectionstr += name;
  cout << "Processing Collection " << collectionstr << endl;
  ifstream from(collectionstr.c_str());
  Collection l;
  from >> l;
  return l;
}


string
makeName(string destName, const char *ext)
{
  string res;
  res += MARSYAS_MFDIR;
  res += "/featurefiles/";
  res += destName;
  res += ext;
  return res;
}


streampos prepareStream(ofstream& os)
{
  streampos pos;
  pos = os.tellp();
  // create space for header
  unsigned int i;
  for (i=0; i < 6; i ++)
    os << "                           " << endl;
  return pos;
}


void writeHeaderFeatures(ofstream& os, streampos pos, string destName, int rows, int cols)
{
  os.seekp(pos);
  os << "# MARSYAS Features" << endl;
  os << "# name: " << destName << endl;
  os << "# type: matrix" << endl;
  os << "# rows: " << rows << endl;
  os << "# columns: " << cols << endl;
}


void writeHeaderLabels(ofstream& os, streampos pos, string destName, int rows, int cols)
{
  os.seekp(pos);
  os << "# MARSYAS Labels" << endl;
  os << "# name: " << destName << endl;
  os << "# type: matrix" << endl;
  os << "# rows: " << rows << endl;
  os << "# columns: " << cols << endl;
}


int myextract(string collection, string destName, string extractor, int winSize,
            int hopSize)
{
  ExtractorDictionary dict;
  string mffName = makeName(destName, ".mff");
  string mlfName = makeName(destName, ".mlf");
  string mdfName = makeName(destName, ".mdf");

  ofstream fos(mffName.c_str());
  streampos fpos = prepareStream(fos);
  ofstream los(mlfName.c_str());
  streampos lpos = prepareStream(los);
    
  int trows = 0;			// total number of rows
  int rows = 0;
  int cols = 0;
  int lcols = 2;			// label cols
  int count =0;
  unsigned int i;

  Collection l = readCollection(collection);
  cout << "read " << collection << endl;      
  string l0name = collection;
  l0name = l0name.substr(0, l0name.size()-3); // remove .mf
  labelNames_.push_back(l0name);
  MSignal *src;
  for (i=0; i<l.size(); i++)
  {
    string fromName = l.entry(i);
    Util util;
    src = util.sfopen(l.entry(i), MRS_SF_READ);
    src->initWindow(winSize, hopSize, 0, 0);
  
    if (extractor == "LPC")
        src->initWindow(512, 128, 0, 0);
    if (extractor == "BEAT") 
        src->initWindow(65536, 65536/2,0,0);
    if (extractor == "DWTC")
        src->initWindow(16384, 512, 0, 0);
    if (extractor == "MPITCH")
        src->initWindow(512, 512, 0, 0);
        
    ExtractorDictionary dict(src);
    Extractor* extr = dict.getExtractor(extractor);
    FeatMatrix* featMatrix = dict.getFeatMatrix(extractor); 
    extr->extract(*featMatrix);
    featMatrix->setName(fromName);
    featMatrix->printHeader(false);
    fos << (fmatrix)(*featMatrix);
    
    imatrix labels(featMatrix->rows(), 2);
    unsigned int r;
    labels.printHeader(false);
    for (r=0; r < labels.rows(); r++)
    {
      labels(r,0) = 0;
      labels(r,1) = count;
    }
    los << labels; 
  
    featNames_ = featMatrix->featNames();
  
    delete src; 
    rows = featMatrix->rows();
    cols = featMatrix->cols();

	trows += rows;
	count ++;
	cerr << "Done with " << l.entry(i) << endl;
  }

  l = readCollection(collection);
  cout << "read " << collection << endl;      
  for (i=0; i<l.size(); i++)
  {
    string l1name = l.entry(i);
	labelNames_.push_back(l1name);
  }
  
  cout << "Wrote " << mffName << endl;

  // Write description file
  ofstream dos(mdfName.c_str());
  dos << "# MARSYAS Description\n" << endl;
  
  unsigned int featSize;
  featSize = featNames_.size();
  unsigned int labelSize;
  labelSize = labelNames_.size();

  dos << "# Feature names " << endl;
  dos << featSize << endl << endl;
  for (i=0; i < featSize; i++)
    dos << featNames_[i] << endl;
  dos << endl;
  dos << "# Label names " << endl;
  dos << labelSize << endl;
  for (i=0; i < labelSize; i++)
    dos << labelNames_[i] << endl;
  
  // write .mff header 
  writeHeaderFeatures(fos, fpos, destName, trows, cols);
  
  // write .mlf header 
  writeHeaderLabels(los, lpos, destName, trows, lcols);
  
  return 0;
}
