/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class ClassifierDictionary.h
    \brief Dictionary of supported extractors

    ClassifierDictionary returns classifiers
based on a string argument which is the classifier name. 
*/


#if !defined(__ClassifierDictionary_h)
#define __ClassifierDictionary_h


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "Classifier.h"
#include "Gaussian.h" 
#include "GMM.h" 
#include "KNN.h" 

#include <iostream.h>
#include <map>
using namespace std;

 
class ClassifierDictionary
{
private:
  /* comparison function for string associative array */
  struct ltstr
  {
    bool operator()(const char* s1, const char* s2) const
    {
      return strcmp(s1, s2) < 0;
    }
  };
  /* Classifiers */ 
  Gaussian* gs_;
  GMM* gmm1_;
  GMM* gmm2_;
  GMM* gmm3_;
  GMM* gmm4_;
  GMM* gmm5_;
  KNN* knn1_;
  KNN* knn3_;
  KNN* knn5_;
  
  vector<string> classifierNames_;
  map<const char *, Classifier *, ltstr> classifiers_;  
  
public:
  ClassifierDictionary();
  ~ClassifierDictionary();
  void prepareNames();
  Classifier* getClassifier(string classifierstr);
  friend ostream& operator<<(ostream&, const ClassifierDictionary&);
};




 
#endif


	

	
