/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Clusterer.h
    \brief Base class for clusterers

    Base class that all clusterers inherit. Unlike Classifiers 
that use supervised learning Clusteres use unsupervised learning. 
Therefore the feature vectors are unlabeled and the only optional 
input is the number of desired clusters. 
*/



#if !defined(__Clusterer_h)
#define __Clusterer_h

#include <stdio.h> 
#include <string.h>
#include "fvec.h"
#include "ivec.h"
#include "FeatMatrix.h"
#include "Collection.h"
#include "Util.h" 

#ifndef WIN32
#include "Audio.h"
#else
#include "AudioWindows.h"
#endif

class Clusterer
{
protected:
  unsigned int classSize_;
  vector<string> classNames_;
  unsigned int featSize_;
  vector<string> featNames_;
  string type_;
  string name_;
  FeatMatrix* trainMatrix_;
  unsigned int numClusters_;
public:
  Clusterer();
  virtual ~Clusterer();
  void init(unsigned int numClusters);
  virtual void seed(FeatMatrix* fmatrix) = 0;
  inline float mahalanobis(fvec& vec1, const fvec& vec2, const fvec& covars);
  virtual void cluster(FeatMatrix* fmatrix) = 0;
  void clusterCollection(string fileliststr, string namesstr);
  //virtual void mask(ivec& vec) = 0;
};


inline float 
Clusterer::mahalanobis(fvec& vec1, const fvec& vec2, const fvec& covars)
{
  unsigned int i;
  float sq_sum = 0.0;
  unsigned int size = vec1.size();
  float diff;
  for (i=0; i<size; i++)
    {
      diff = vec1(i) - vec2(i);
      sq_sum += diff * covars(i) * diff;
    }
  return sq_sum;
}


  
#endif
