/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/**
   \class Collection
   \brief List of files

   List of files used for batch processing.
*/



#include "Collection.h"


Collection::Collection()
{
  collectionList_.reserve(1000);
}



Collection::~Collection()
{
}

void 
Collection::setName(string name) 
{
  name_ = name;
}


void 
Collection::read(string filename)
{
  ifstream is(filename.c_str());
  is >> (*this);
}


void 
Collection::write(string filename)
{
  ofstream os(filename.c_str());
  os << (*this) << endl;
}


ostream& 
operator<<(ostream& o, const Collection& l)
{
  o << "# MARSYAS Collection " << endl;
  o << "# name = " << l.name_ << endl << endl;
  for (unsigned int i=0; i < l.collectionList_.size(); i++)
    {
      o << l.collectionList_[i] << endl;
    }
  o << endl;
  return o;
}



unsigned int 
Collection::size()
{
  return collectionList_.size();
}


string
Collection::name()
{
  return name_;
}

void 
Collection::add(string entry)
{
  collectionList_.push_back(entry);
}


void 
Collection::shuffle()
{
  unsigned int i;
  unsigned int size = collectionList_.size();
  unsigned int rind;
  string temp;
  
  for (i=0;  i < size; i++)
    {
      rind = (unsigned int)(((float)rand() / (float)(RAND_MAX))*size);
      temp = collectionList_[i];
      collectionList_[i] = collectionList_[rind];
      collectionList_[rind] = temp;
    }
}


string 
Collection::entry(unsigned int i)
{
  return collectionList_[i];
}



istream& 
operator>>(istream& i, Collection& l)
{
  string str0, str1, str2;
  string fileEntry;
  
  i >> str0;
  i >> str1;
  i >> str2;
  
  if ((str0 != "#") || ( str1 != "MARSYAS") || (str2 != "Collection"))
    {
      cout << "Problem reading collection from istream" << endl;
      cout << "str0 = " << str0 << endl;
      return i;
    }

  i >> str0;
  i >> str1;
  i >> str2;
  i >> l.name_;
  
  
  while (i >> fileEntry)
    {
      char c = fileEntry[0];
      if (c !='#')
	l.collectionList_.push_back(fileEntry);
    }
  return i;
}

