/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class  CollectionShuffler
   \brief  Interface for shufflers

   Abstract base class used as interface for shufflers. CollectionShufflers 
   as input a FeatMatrix and produce a temporal reording (shuffling) 
   of the file and returns it as a ivec called mapping.
*/


/* CollectionShuffler:

*/


#if !defined(__CollectionShuffler_h) 
#define __CollectionShuffler_h

#include <string.h>
#include "FeatMatrix.h"
#include "fvec.h"
#include "Shuffler.h"

class CollectionShuffler: public Shuffler
{
protected:
  string type_;
  FeatMatrix* cmatrix_;
public:
  CollectionShuffler();
  ~CollectionShuffler();
  inline float mahalanobis(fvec& vec1, const fvec& vec2, const fvec& covars);
  void init(FeatMatrix* cmatrix);
  void shuffle(FeatMatrix& fmatrix, ivec &mapping, int winNum);
};



inline float 
CollectionShuffler::mahalanobis(fvec& vec1, const fvec& vec2, const fvec& covars)
{
  unsigned int i;
  float sq_sum = 0.0;
  unsigned int size = vec1.size();
  float diff;
  
  for (i=0; i<size; i++)
    {
      diff = vec1(i) - vec2(i);
      sq_sum += (diff * covars(i) * diff);
    }
  return sq_sum;
}


#endif

   
