/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/



/**
   \class DFCC:
   \brief Mel-frequency cepstral coefficients

   Mel-frequency cepstral coefficients. Features commonly used 
in Speech Recognition research. Code based on the Matlab 
implementation of Malcolm Slaney. 
*/

#if !defined(__DFCC_h)
#define __DFCC_h

#include "System.h"
#include "MagFFT.h"
#include "fvec.h"
#include "fmatrix.h"
#include <math.h>
#include "Hamming.h"
#include "SpectralPeak.h"
#include "Pitch.h"

class DFCC: public System
{
private:
	Pitch *pitch_;
	float hamsum_;
	float hamsum2_;
	int maxorder_;
	int maxpartials_;
	bool automatic_;
	fmatrix M;
	fmatrix A;
	fmatrix L;
	fmatrix Li;
	fvec peaks_;
	fvec amps_;
	fvec spectrum_;
	fvec noise_;
	fvec pit;
	fvec b;
	fvec d;
	fvec y;
	fvec r;
	fvec a;
	fvec f;
	fvec h;
	void init(void);

public:
  DFCC();
  DFCC(unsigned int maxpartials, unsigned int order);
  DFCC(unsigned int inSize, unsigned int maxpartials, unsigned int order);
  ~DFCC();
	void wls(int partials, int order, fvec& freq, fvec& amp, fvec& weight, fvec& o, float g);
  void set(unsigned int inSize, unsigned int order);
  float get(fvec& envelope, float freq);
  void process(fvec& in, fvec& out);
	float pitch();
	fvec& noise();
};


#endif 
