/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class DWTC
   \brief Features based on the DWT

   Features based on the Discrete Wavelet Transform. 
*/


#include "DWTC.h"

DWTC::DWTC()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = 24;
  norm_ = NULL;
  daub4_ = NULL;
  pyramid_ = NULL;
  init();
}

DWTC::DWTC(MSignal *src)
{
  inSize_ = src->winSize();
  outSize_ = 24;
  norm_ = NULL;
  daub4_ = NULL;
  pyramid_ = NULL;
  init();
}


void 
DWTC::init()
{
  norm_ = new NormRMS(inSize_);
  daub4_ = new Daub4(inSize_, WVL_FORWARD);
  res_.create(inSize_);
  pyramid_ = new WaveletPyramid(inSize_, daub4_, WVL_FORWARD);
  featSize_ = 24;
  unsigned int i;
  for (i=0; i < featSize_/2; i++) 
    {
      featNames_.push_back("DWTmean");
      featNames_.push_back("DWTstd");
    }
}




DWTC::~DWTC()
{
  delete norm_;
  delete pyramid_;
  delete daub4_;
}


void 
DWTC::process(fvec& in, fvec& out) 
{
  unsigned int i;
  assert((in.size() == inSize_) && (out.size() == outSize_));    
  
  norm_->process(in,in);
  pyramid_->process(in, res_);
  
  for (i=0; i < outSize_; i++) 
    {
      out(i) = 0;
    }
  unsigned int base = 4;
  unsigned int level = 9;
  unsigned int hlevel, llevel;
  float zcr;

  for (level = 1; level < 13; level++)
    {
      hlevel = base << level;
      llevel = base << (level-1);          
      fvec temp(hlevel - llevel);
      
      for (i=llevel; i < hlevel; i++)
	{
	  temp(i-llevel) = res_(i);
	}

      out((level-1)*2+1) = temp.std();
      temp.abs();
      out((level-1)*2) = temp.mean();
    }
  out(20) = 0.0;
  out(21) = 0.0;
  out(22) = 0.0;
  out(23) = 0.0;
}


	

	
