/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/**
   \class  FeatExtractor
   \brief  Feature calculation extractor

   This extractor is initialized with a MSignal and a System that calculate 
feature values given a window of data. The extractor iterates through the given MSignal applying the System to each window of data and possibly storing their resulting feature vectors to a FeatMatrix. Optional plotting is supported with mode_.
*/





#include "FeatExtractor.h"

FeatExtractor::FeatExtractor(MSignal* in, System* feat):Extractor(in)
{
  feat_ = feat;
  in_.allocate(feat->inSize());
  out_.allocate(feat->outSize());
}

FeatExtractor::~FeatExtractor()
{
}





void 
FeatExtractor::extract(unsigned int start, unsigned int end, 
		      FeatMatrix& fmatrix) 
{
  int load_matrix = 1;
  if ((fmatrix.rows() != end-start)||(fmatrix.cols() != out_.size()))
    load_matrix = 0;
  
  unsigned int t;
  unsigned int p=0;
  fmatrix.setFeatNames(feat_->featNames());
  for (t=start; t< end; t++)
    {
      src_->get(in_, t);
      feat_->process(in_, out_);
      if (load_matrix) 
	{
	  fmatrix.row(t-start, out_);
	}
      if (t % ((end-start)/5) == 0)
	{
	  cerr << p * 20  << "% ";
	  p++;
	} 
    } 
  cerr << "\nFeature Extractor done" << endl;
}

void
FeatExtractor::extract(unsigned int start, unsigned int end)
{
  unsigned int t;
  for (t=start; t< end; t++)
    {
      src_->get(in_, t);
      feat_->process(in_, out_);
      if (t % ((end-start)/10))
	{
	  cerr << (end-start)/t * 100.0 << "% processed" << endl;
	}      
    }
}

