/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/




/**  
     \class Filter
     \brief Filter a MSignal 

     Filters a MSignal using the coefficients stored in a
FilterCoeffs object. The filter is implemented as a direct form II
structure. This is a canonical form which has the minimum number of
delay elements.  
*/



#if !defined(__Filter_h)
#define __Filter_h

#include "System.h"

class Filter: public System
{
protected:
  unsigned int norder_;			// Numerator order
  unsigned int dorder_;			// Denominator order
  unsigned int order_;
  string type_;
  fvec state_;
  int stateSize_;
  float fgain_;
  fvec ncoeffs_;
  fvec dcoeffs_;

public:
  Filter(fvec ncoefs, fvec dcoefs, string type);
  ~Filter();
  void process(fvec& in, fvec& out);
  friend ostream& operator<<(ostream&, const Filter&);
  friend istream& operator>>(istream&, Filter&);

  unsigned int order();
  unsigned int norder();
  unsigned int dorder();
  

  void read(string filename);
  void write(string filename);

};

	
#endif
 
