/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Gaussian.h
    \brief Gaussian classifier

    Gaussian classifier is a classifier where each class is modelled as 
a single multidimensional Gaussian distribution. The features 
are assumed to be uncorrelated and a diagonal covariance matrix 
is used.
*/


#if !defined(__Gaussian_h)
#define __Gaussian_h


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <iostream.h>
using namespace std;

#include "FeatMatrix.h"
#include "Classifier.h"
 
class Gaussian:public Classifier
{
private:
  fmatrix vars_;
  fmatrix covars_;
  fmatrix means_;

  

public:
  Gaussian();
  ~Gaussian();
  void hardwire();
  void train(FeatMatrix* fmap, unsigned int lcol); // Train classifier
  unsigned int classify(fvec& vec);
  void read(string filename);
  void write(string filename);
  void info();
  friend ostream& operator<<(ostream&, Gaussian&);
  friend istream& operator>>(istream&, Gaussian&);
};




 
#endif


	

	
