/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class  GenreExtractor
   \brief  Single Vector feature extractor for genre information

   Combination of SVFFT (spectral timbral features), 
BEAT (beat related information calculated using the DWT). 
and MPITCH (pitch related information)
*/





#if !defined(__GenreExtractor_h)
#define __GenreExtractor_h

#include "Extractor.h"
#include "GenreExtractor.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "Centroid.h"
#include "DB.h"
#include "Series.h"
#include "RMS.h"
#include "WaveletPyramid.h"
#include "Daub4.h"
#include "FullWaveRectifier.h"
#include "OnePole.h" 
#include "Norm.h"
#include "AutoCorrelationFFT.h" 
#include "DownSampler.h" 
#include "PeakPicker.h" 
#include "NormRMS.h"
#include "Spectral.h"
#include "MultiPitch.h"
#include "BeatExtractor.h"
#include "SvFeatExtractor.h"
#include "MultiPitchExtractor.h"
#include "MemMFCC.h"

class GenreExtractor:public Extractor
{
protected:

  BeatExtractor*  beatExtractor_;
  MultiPitchExtractor*  mpitchExtractor_;
  SvFeatExtractor*  svspectralExtractor_;
  SvFeatExtractor* svmfccExtractor_;
  Spectral* spectral_;
  MemMFCC* mfcc_;
  
  unsigned waveletSize_;
  unsigned fftSize_;
  unsigned int dsfactor_;

  unsigned int featSize_;
  vector<string> featNames_;
  
  
 
public:
  GenreExtractor(MSignal* in);
  ~GenreExtractor();
  void extract();
  void extract(unsigned int start, unsigned int end);
  void extract(unsigned int start, unsigned int end, FeatMatrix& fmatrix);
  unsigned int outSize();
};


#endif	


	
