/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/



/* Main Structures used by Gmpg (MP3 decoder) */
/* Written by George Tzanetakis (gtzan@cs.princeton.edu) */

#if !defined(__Gmpg_main_h)
#define __Gmpg_main_h


#define MAX_FRAME_SIZE 3840
#define MAX_CHANNEL 2
#define MAX_READ_LIST 4
#define MAX_SCALE_FACTORS 39
#define IBLEN 576


#define         PI4                     M_PI/4
#define         PI64                    M_PI/64


/* Fixed size buffers used for circular list of 
read buffers. Layer III uses bit-reservoir technique 
where data corresponding to a frame in time can 
span multiple previous frames. */
// static unsigned char g_data_buffers[MAX_READ_LIST][MAX_FRAME_SIZE];
// static unsigned char g_decode_buffer[MAX_FRAME_SIZE];



typedef struct Header
{
  /* These fields are explained in the MPEG ISO spec for audio 
     and are read from the audio frame header
  */
  int mpeg_type;
  int layer;
  int bitrate_index;
  int sampling_frequency;
  int error_protection;
  int padding;
  int extension;
  int mode;
  int mode_ext;
  int copyright;
  int original;
  int emphasis;
  int size;				/* Size in bytes */
} Header;


/* Structure for holding the frequency position and bandwidth in frequency 
lines of each subband. */

struct bandInfoStruct {
  int longIdx[23];
  int longDiff[22];
  int shortIdx[14];
  int shortDiff[13];
};







/* Alias reduction tables */
//static double alias_coefs[8] = {-0.6, -0.535, -0.33, -0.185, -0.095, -0.041, -0.0142, -0.0037};
//static int pretab[22] = {0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,2,2,3,3,3,2,0};


/* Global static buffers used for decoding */
// static double g_N[64][32];
// static double g_V[1024];





typedef struct Side_Info_III
{
					        /* # bits */
  unsigned main_data_begin;		        /* 8   */
  unsigned private_bits;		        /* 1,2 */
  unsigned part2_3_length[MAX_CHANNEL];	        /* 12  */
  unsigned big_values[MAX_CHANNEL];             /* 9   */
  unsigned global_gain[MAX_CHANNEL];            /* 8   */
  unsigned scalefac_compress[MAX_CHANNEL];      /* 9   */
  unsigned window_switching_flag[MAX_CHANNEL];  /* 1   */
  unsigned preflag[MAX_CHANNEL];	        /* scalefactors derived*/

  /* Window switching == 1 */ 
  unsigned block_type[MAX_CHANNEL];
  unsigned mixed_block_flag[MAX_CHANNEL];
  unsigned table_select[MAX_CHANNEL][3];
  unsigned subblock_gain[MAX_CHANNEL][3];
  
  /* Window switching == 0 */
  unsigned region0_count[MAX_CHANNEL];
  unsigned region1_count[MAX_CHANNEL];
  unsigned scalefac_scale[MAX_CHANNEL];
  unsigned count1table_select[MAX_CHANNEL];

  /* Extra fields added for easier region handling */
  unsigned region1_start[MAX_CHANNEL];
  unsigned region2_start[MAX_CHANNEL];
} Side_Info_III;


typedef struct Granule_Info
{
  /* main fields described in the standard spec */
  int scalefac_l[MAX_CHANNEL][23];
  int scalefac_s[MAX_CHANNEL][3][13];
  int scale_factors[MAX_SCALE_FACTORS];
  int is[IBLEN];
  int is_sfb;

  /* Temporary auxilliary decoding buffers */
  double temp_xr[IBLEN];
  double xr_l[IBLEN];
  double xr_r[IBLEN];
  double sr[32*18];
  double sl[32*18];
  double result_l[32*18];
  double result_r[32*18];
} Granule_Info;



/* Circular list to handle bit reservoir */

typedef struct Read_List
{
  unsigned char *buffer[MAX_READ_LIST];
  unsigned int size[MAX_READ_LIST];
  unsigned char *data_begin[MAX_READ_LIST];
  unsigned int data_size[MAX_READ_LIST];
  int cur_pos;
} Read_List;



/* Structure used for bitstream reading and huffman decoding */
typedef struct Bs_ptr
{
  unsigned char *decode_buf;
  unsigned char *word_ptr;
  unsigned int bit_index;
} Bs_ptr;

/* Structure for holding auxilliary decoding information */ 
/* Basically precomputed tables that apply for each frame 
   for faster decoding */ 

typedef struct Aux_info
{
  double is_pow[8207];
  double imdct_win[4*36];
  double imdct_cos[4*36];
  double imdct_cos1[12*6];
  double imdct_cos2[36*18];

} Aux_info;






/* Complete decoder structure. Contains all the necessary structures 
for decoding. */

typedef struct MPEG_decoder
{
  Granule_Info granule_info;
  Header header;
  Side_Info_III side_info_III;
  Read_List read_list;
  Bs_ptr bs_ptr;
  Aux_info aux_info;
} MPEG_decoder;




#endif
