
/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/** 
    \class KNN.h 
    \brief K nearest neighbors classifier 

    The K nearest neighbors classifier (KNN) is a non-parametric 
classifier where each point is classified according to the majority 
class label vote of its k nearest neighbors. 

*/

#include "KNN.h"


KNN::KNN(unsigned int k) 
{
  k_ = k;
  lcol_ = 0;
}


KNN::~KNN()
{
}


void 
KNN::train(FeatMatrix* featMatrix, unsigned int lcol)
{
  unsigned int c;
  lcol_ = lcol;
  trainMatrix_ = featMatrix;
  classSize_ = trainMatrix_->labelSize(lcol);
  mask_.create(classSize_);
  mask_.setval(1);
  cols_ = trainMatrix_->cols();
  rows_ = trainMatrix_->rows();
  votes_.create(classSize_);
  mins_.create(k_);
  minsPos_.create(k_);
  vars_.create(cols_);
  covars_.create(cols_);
  vars_ = trainMatrix_->varRow();
  for (c=0; c < cols_; c++)
    {
      if (vars_(c) <= 0.000001) 
	{
	  covars_(c) = 1.0;		// No variance
	}
      else
	{
	  covars_(c) = 1.0 / vars_(c);
	}
    }
  classNames_ = trainMatrix_->labelNames();
  featNames_ = trainMatrix_->featNames();
}

unsigned int
KNN::classify(fvec& vec)
{
  unsigned int cl,r,k;
  unsigned int classNum=0;
  unsigned int max = 0;
  float dist;
  
  votes_.setval(0);
  mins_.setval(MAX_INT);
  
  float mymin = MAX_INT;
  unsigned int myminPost = 0;
  
  
  for (r=0; r < rows_; r++)
    {
      dist = mahalanobis(vec, trainMatrix_->row(r), covars_);
      for (k=0; k < k_; k++)
	if ((dist < mins_(k)) && (mask_(trainMatrix_->labelNum(r,lcol_)) ==1))
	{
	    mins_(k) = dist;
	    minsPos_(k) = r;
	    break;
	}
    }
  
  // cout << "Dist " << mins_(0) << endl;
  for (k=0; k< k_; k++)
    {
      classNum = trainMatrix_->labelNum(minsPos_(k),lcol_);
      votes_(classNum)++;
    }
  
  for (cl=0; cl < classSize_; cl++)
    {
      if ((unsigned int)(votes_(cl)) > max) 
	{
	  max = votes_(cl);
	  classNum = cl;
	}
    }
  return classNum;
}





