/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class K_MEANS.h
    \brief K-means clustering

    Clustering using the k-means algorithm. 
*/


#include "K_MEANS.h"

K_MEANS::K_MEANS():Clusterer()
{
}

K_MEANS::~K_MEANS()
{
}

void 
K_MEANS::seed(FeatMatrix* fmatrix) 
{
  unsigned int cl;
  int num = fmatrix->rows() / numClusters_-1;
  for (cl=0; cl < numClusters_; cl++)
    {
      fmatrix->setLabelRange(cl * num, (cl+1) * num, 0, cl);
    }
  

}

float 
K_MEANS::sqr_error(fvec& vec1, fvec& vec2)
{
  float error = 0.0;
  assert(vec1.size() == vec2.size());
  unsigned int i;
  unsigned int size = vec1.size();
  float v1,v2;
  
  for (i=0; i < size; i++)
    {
      v1 = vec1(i);
      v2 = vec2(i);
      error += ((v1 - v2) * (v1 - v2));
    }
  return error;
}


float 
K_MEANS::means_error(fmatrix& m1, fmatrix& m2)
{
  unsigned int cl;
  float error=0.0;
  fvec v1,v2;
  v1.create(m1.cols());
  v2.create(m2.cols());
  for (cl = 0; cl < classSize_; cl++)
    {
      v1 = m1.row(cl);
      v2 = m2.row(cl);
      error += sqr_error(v1,v2);
    }
  return error;
}


void 
K_MEANS::cluster(FeatMatrix* featMatrix) 
{
  float thres,error, perror = 0;
  unsigned int cl;
  char buffer[100];
  vector<string> classNames;
  classNames.reserve(numClusters_);
  for (cl = 0; cl < numClusters_; cl++)
    {
      sprintf(buffer, "%s%d", "cluster", cl);
      classNames.push_back(buffer);
    }
  featMatrix->setLabelNames(classNames);
  seed(featMatrix);
  Gaussian gaussian;
  fmatrix oldmeans;
  fmatrix oldvars;
  fmatrix means;
  fmatrix vars;
  
  
  classSize_ = featMatrix->labelSize(0);
  featSize_ = featMatrix->cols();
  oldmeans.create(classSize_, featSize_);
  oldvars.create(classSize_, featSize_);
  means.create(classSize_, featSize_);
  vars.create(classSize_, featSize_);

  thres = 0.00001;
  error = 100.0;
  unsigned int osc = 0;
  unsigned int iterations=0;
  
  while (error > thres)
    {
      cout << "Error " << error << endl;
      gaussian.train(featMatrix,0);
      featMatrix->classStat(0,oldmeans, oldvars);
      gaussian.classifyFeatMatrix(*featMatrix);
      featMatrix->classStat(0,means, vars);
      error = means_error(oldmeans, means);
      if (error == perror) 
	osc++;
      perror = error;
      if (osc == 20) 
	break;
      iterations++;
    }
  if (osc == 20) 
    cout << "Clustering terminated because of oscillation\n" << endl;
  cout << "K_MEANS iterations = " << iterations << endl;
  cout << means << endl;
}
