/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class K_MEANS.h
    \brief K-means clustering

    Clustering using the k-means algorithm. 
*/



#if !defined(__K_MEANS_h)
#define __K_MEANS_h

#include <stdio.h> 
#include <string.h>
using namespace std;

#include "fvec.h"
#include "ivec.h"
#include "FeatMatrix.h"
#include "Clusterer.h"
#include "Gaussian.h"
#include "fmatrix.h"

class K_MEANS: public Clusterer
{
protected:
  unsigned int classSize_;
  vector<string> classNames_;
  unsigned int featSize_;
  vector<string> featNames_;
  string type_;
  string name_;
  FeatMatrix* trainMatrix_;
  float sqr_error(fvec& vec1, fvec& vec2);
  float means_error(fmatrix& m1, fmatrix& m2);
public:
  K_MEANS();
  virtual ~K_MEANS();
  void cluster(FeatMatrix* fmatrix);
  void seed(FeatMatrix* fmatrix);
  //virtual void mask(ivec& vec) = 0;
};




  
#endif
