/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/





/**
   \class LPCResyn:
   \brief Mel-frequency cepstral coefficients

   Mel-frequency cepstral coefficients. Features commonly used 
in Speech Recognition research. Code based on the Matlab 
implementation of Malcolm Slaney. 
*/



#include "LPCResyn.h"

LPCResyn::LPCResyn()
{
  inSize_ = DEFAULT_ORDER+1;
  outSize_ = DEFAULT_WIN_SIZE;
  order_ = DEFAULT_ORDER;
  Zs_.create(order_);
  init();
}

LPCResyn::LPCResyn(unsigned int inSize, unsigned int hopSize, unsigned int order)
{
  inSize_ = inSize;
  outSize_ = hopSize;
  order_ = order;
  hopSize_ = hopSize;
  Zs_.create(order_);
  init();

}

LPCResyn::~LPCResyn()
{
}

void
LPCResyn::init()
{
}







void 
LPCResyn::process(fvec& in, fvec& out)
{
  assert((in.size() == inSize_) && (out.size() == outSize_));
  float input;
  float pitch;
  float output;
  unsigned int i,j;
  
  input = in(inSize_-1);
  pitch = in(inSize_-2);
  ticker_  = pitch;
  for (i=0; i < outSize_; i++)
    {
      output = 0.0;
      if (pitch == 0.0) 
	output = input * 20.0 * (1.0 - rand() * ONE_OVER_RANDLIMIT);
      else
	{
	  ticker_--;
	  if (ticker_ <= 0)
	    {
	      output = input * pitch * 3.0;
	      ticker_ = pitch;
	    }
	}
      
      
      for (j=0; j < order_; j++)
	{
	  output += Zs_(j) * in(j);
	}
      for (j= order_-1; j>0; j--)
	Zs_(j) = Zs_(j-1);
      Zs_(0) = output;
      //printf("output = %f\n", output);
      out(i) = output;
    }
  
  
}

