/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class MagDB
   \brief FFT magnitude in DB

   Calculate the FFT magnitude in DBs of the input fvec. 
Example of using the Series class. 
*/



#include "MagDB.h"

MagDB::MagDB()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = DEFAULT_WIN_SIZE;
}

MagDB::MagDB(unsigned int inSize, unsigned int zeroSize, 
			 float dbfloor)
{
  cerr << "MagDB constructor" << endl;
  
  inSize_ = inSize;
  
  hamming_ = new Hamming(inSize_, zeroSize_);
  magfft_  = new MagFFT(inSize_);
  db_      = new DB(magfft_->outSize(), dbfloor);
  
  outSize_ = magfft_->outSize();

  vector<System *> featArray;
  featArray.push_back(hamming_);
  featArray.push_back(magfft_);
  featArray.push_back(db_);
  series_ = new Series(featArray);
}

MagDB::~MagDB()
{
  delete hamming_;
  delete magfft_;
  delete db_;
  delete series_;
}


void 
MagDB::process(fvec& in, fvec& out) 
{
  if ((in.size() != inSize_) || (out.size() != outSize_))
    {
      cerr << "Warning: MagDB::process: inSize_ and input window size do not agree" << endl;
    }
  series_->process(in, out);
}


	

	
