/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class MagDB
   \brief FFT magnitude in DB

   Calculate the FFT magnitude in DBs of the input fvec. 
Example of using the Series class. 
*/

#if !defined(__MagDB_h)
#define __MagDB_h

#include "System.h"	
#include "DB.h"
#include "MagFFT.h"
#include "Hamming.h"
#include "Series.h"
#include "Centroid.h"

class MagDB: public System
{
private:
  unsigned int zeroSize_;
  Series* series_;
  Hamming* hamming_;
  MagFFT* magfft_;
  DB* db_;
  Centroid* centroid_;
  
  
public:
  MagDB();
  MagDB(unsigned int inSize, unsigned int zeroSize, float dbfloor);
  ~MagDB();
  void process(fvec& in, fvec& out);
};

#endif

	
