/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class Mp3FileIO
   \brief SoundFileIO for mp3 compressed soundfiles
   
*/



#include "Mp3FileIO.h"



Mp3FileIO::Mp3FileIO(string filename, int mode):SoundFileIO(filename, mode)
{
  gmpg_ = NULL;
}


Mp3FileIO::~Mp3FileIO()
{
  delete gmpg_;
}

int 
Mp3FileIO::check_type()
{
  FileName fname(filename_);
  string mp3ext = "mp3";
  if (!sfp_)
    {
      cerr << "Mp3FileIO: File " << filename_ << " does not exist" << endl;
      return MRS_ERROR;
    }
  if (fname.ext() == mp3ext)
    return MRS_OK;
  else
    return MRS_ERROR;
}



void 
Mp3FileIO::init()
{
  cout << "Mp3FileIO::init called" << endl;
  if (mode_ == MRS_SF_READ)
    {
      gmpg_ = new Gmpg(filename_, sfp_);
      gmpg_->go();
      winSize_ = DEFAULT_WIN_SIZE;
      hopSize_ = DEFAULT_HOP_SIZE;
      skipSize_ = DEFAULT_SKIP_SIZE;
      zeroSize_ = DEFAULT_ZERO_SIZE;
      channels_ = 1;
      sizeSamples_ = gmpg_->getSize();
      sizeSeconds_ = sizeSamples_ / srate_;
      comments_    = "Mp3 file";
      type_ = "Mp3 file";
    }
  else if (mode_ == MRS_SF_WRITE)
    {
      cout << "WRITE MODE NOT SUPPORTED FOR MP3 FILES" << endl;
      return;
    }
}


void
Mp3FileIO::get(fvec &win, unsigned long index)
{
  unsigned long start = index * hopSize_ + skipSize_;
  gmpg_->get(win, start);
}


void 
Mp3FileIO::put(fvec &win, unsigned long index)
{
  cout << "WRITE MODE NOT SUPPORTED FOR MP3 FILES\n" << endl;
  if (win.size() != winSize_) 
    {
      cerr << "Mp3FileIO::put incompatible window sizes\n" << endl;
      exit(1);
    }
  
}

  
  


