/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class MultiPitch
    \brief Multiple pitch extractor

    Multiple pitch extractor based on the paper by 
T.Tolonen and M.Karjalainen "A computationally efficient multipitch analysis 
model IEEE Trans.on Speech and Audio Processing vol 8, num 6. 2000.
*/

#if !defined(__MultiPitch_h)
#define __MultiPitch_h

#include "System.h"	
#include "Filter.h"
#include "HalfWaveRectifier.h"
#include "AutoCorrelationFFT.h"
#include "PeakEnhancer.h"
#include "math.h"
#include "AMDF.h"

class MultiPitch: public System
{
  Filter* lpf1_;
  Filter* lpf2_;
  Filter* hpf1_;
  HalfWaveRectifier* hwr_;
  AutoCorrelationFFT* autocor_;
  AMDF* magdif_;
  PeakEnhancer* pe2_;
  PeakEnhancer* pe3_;
  PeakEnhancer* pe4_;
  
  vector<string> featNames_;
  fvec lowp_;
  fvec highp_;
  fvec sum_;
  fvec lastthree;
public:
  MultiPitch();
  ~MultiPitch();
  MultiPitch(unsigned int inSize);
  void init();
  void process(fvec& in, fvec& out);

};

#endif
