/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class OnePole
    \brief OnePole filter

   OnePole Filter. 
   Output is normalized by setting the
gain to (1.0-a). 
*/


 



#include "OnePole.h"


OnePole::OnePole()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = DEFAULT_WIN_SIZE;
  a_ = 0.9;
  gain_ = 1.0 -a_;
}

OnePole::OnePole(unsigned int inSize,float a)
{
  inSize_ = inSize;
  outSize_ = inSize_;
  a_ = a;
  gain_ = 1.0 -a_;
}


OnePole::~OnePole()
{
}


void
OnePole::set(float a)
{
  a_ = a;
  gain_ = 1.0 -a;
}


void
OnePole::process(fvec& in, fvec& out)
{
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  unsigned int size = in.size();
  unsigned int i;
  
  for (i=1; i< size; i++)
    {
      out(i) = gain_ * in(i) + a_ * out(i-1);
    }
}

	
