/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Parallel
    \brief Parallel connection of System objects
    
    Combines a series of System objects to a single System 
corresponding to executing the System objects in Parallel 
on the same input fvec and concatenating their output fvec objects.
*/








#include "Parallel.h"


Parallel::Parallel()
{
  numFeats_ = 0;
  inSize_ =  DEFAULT_WIN_SIZE;
  outSize_ = DEFAULT_WIN_SIZE;
  vectors_ = NULL;
}


Parallel::Parallel(vector<System*> featArray)
{
  unsigned int i,j;
  numFeats_ = featArray.size();
  //numFeats_ = numFeats;
  featArray_ = featArray;
  vectors_ = new fvec[numFeats_];
  inSize_ = featArray_[0]->inSize();
  outSize_ =0;
  for (i=0; i<numFeats_; i++)
    {
      outSize_ += (featArray_[i]->outSize());
      vectors_[i].allocate(featArray_[i]->outSize());
      featSize_ += (featArray_[i]->outSize());
      vector<string> featNames = featArray_[i]->featNames();
      for (j=0; j < featArray_[i]->featSize(); j++)
	featNames_.push_back(featNames[j]);
    }
}


Parallel::~Parallel()
{
  delete [] vectors_;
}


void
Parallel::process(fvec& in, fvec& out)
{
  if ((in.size() != inSize_) || (out.size() != outSize_))
    {
      cerr << "Warning: Parallel::process: inSize_ and input window size do not agree" << endl;
      cerr << "inSize_ = " << inSize_ << endl;
      cerr << "input window size = " << in.size() << endl;
      cerr << "outSize_ = " << outSize_ << endl;
      cerr << "output window size = " << out.size() << endl;
      return;
    }
  unsigned int pos = 0;
  unsigned int vsize = 0;
  for (unsigned int i=0; i<numFeats_; i++)
    {
      featArray_[i]->process(in, vectors_[i]);
      vsize = vectors_[i].size();
      for (unsigned int j=pos; j< pos + vsize; j++)
	{
	  out(j) = (vectors_[i])(j-pos);
	}
      pos += vsize;
    }
}




	
