#include "Pitch.h"

Pitch::Pitch(string model)
{
  inSize_ = DEFAULT_WIN_SIZE;
  model_ = new PitchModel(inSize_);
  outSize_ = 2;
  init();
}

Pitch::Pitch(string model, int inSize)
{
  inSize_ = inSize;
  model_ = new PitchModel(inSize_);
  outSize_ = 2;
  init();
}

Pitch::~Pitch()
{
}


void
Pitch::init()
{
  envelope_ = new DFCC(10, 10);

  featNames_.push_back("pitch");
  featNames_.push_back("mag");
  
  maxpartials_ = 20;
  spectrum_.create(inSize_/2);
  quality_.create(inSize_/2);
  likelihood_.create(inSize_/2);
	hist_.create(inSize_/2);
  partials_.create(50);
	minBin_ = (int)(100*inSize_/44100);
	maxBin_ = inSize_/2 - 1;
	order_ = 20;
	ccfs_.create(order_);
}



void 
Pitch::process(fvec& in, fvec& out) 
{
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  cout << "Pitch" << endl;
  int i, j, k, candidate, min, max, cutoff = 0;
	fmatrix h(maxpartials_, inSize_/2);
  float val;

  //calculate power spectrum, get qualities, and pick partials	
	npartials_ = model_->getPartials(in, spectrum_, quality_, partials_);

  //create histogram f0s accosicated with each partial
	h.setval(0); 
	hist_.setval(0);
	for(i = 0; i < npartials_; i++)
	  for(j = 1; (min=(int)floor(partials_(i)/(j+0.5))) != (max=floor(partials_(i)/(j-0.5))) && j < maxpartials_; j++)
			for(k = min; k <= max; k++) h(j, k) = 1;
	for(i = 0, cutoff = 0; i < inSize_/2; i++)
	{
	  for(j = 1; j < maxpartials_; j++) hist_(i) += h(j, i);
		if(hist_(i) > cutoff) cutoff = hist_(i);
	}
	cutoff = 2 * (int)floor((float)cutoff/3);;
		
	//check each candidate whose histogram value is above the cutoff
	out.setval(0);
	for(candidate = minBin_; candidate < maxBin_; candidate++) if(hist_(candidate) >= cutoff)
	{
		val = getLikelihood(candidate);
		if(val > out(1))
		{
			out(0) = candidate;
			out(1) = val;
		}
	}

	ofstream hist("hist"), like("like"), spect("spectrum"), env("envelope");

	fvec ps(npartials_*2);
	for(i = 0; i < npartials_; i++)
	{
		ps(i) = 2*PI*partials_(i)/inSize_;
		ps(i+npartials_) = spectrum_(partials_(i));
	}
	envelope_->set(npartials_*2, order_);
	envelope_->process(ps, ccfs_);
	
/*	for(i = 0; i < inSize_/2; i++)
	{
	  hist << hist_(i) << endl;
	  like << likelihood_(i) << endl;
		spect << spectrum_(i) << endl;
		env << envelope_->get(ccfs_, i*2*PI/inSize_) << endl;	
	}
	*/
	
}
  

inline float
normpdf(float value, float std, float mean)
{
  return exp(-(value-mean)*(value-mean)/(2*std*std))/(std*sqrt(2*PI));
}


inline float
Pitch::getLikelihood(int candidate)
{
  fvec probs(maxpartials_+1), indexes(maxpartials_+1), parnum(maxpartials_+1), params(3);
	int i, p, dist;
	float offset, prob, fstd = .03, amp;
	float fscale = normpdf(0, fstd, 0), pscale = normpdf(0, maxpartials_, 0);
	
	probs.setval(0);
	indexes.setval(0);
	for(i = 0; i < npartials_; i++)
	{
		p = round((float)partials_(i)/candidate);
		if(p > 0 && p < maxpartials_)
		{
		  offset = ((float)partials_(i)-p*candidate)/(p*candidate);
			prob = normpdf(offset, fstd, 0) / fscale;
			//amp = envelope_->get(ccfs_, 2*PI*partials_(i)/inSize_);
			//prob *= normpdf(amp - spectrum_(partials_(i)), amp/4, amp) / normpdf(amp, amp/4, amp);
			prob *= normpdf(p, maxpartials_, 0) / pscale;
			if(prob > probs(p))
			{
			  probs(p) = prob;
				indexes(p) = 1;
			}
		}
	}
	
	likelihood_(candidate) = 1;
	for(i = 1; i < maxpartials_; i++)
	{
	  if(indexes(i)) likelihood_(candidate) *= probs(i);
 	  else likelihood_(candidate) *= 1.0 - normpdf(i, maxpartials_, 0) / pscale;
	}
	for(i = 0; i < npartials_; i++) if(!parnum(i))
	{
		//amp = envelope_->get(ccfs_, 2 * PI * partials_(i) / inSize_);
		//likelihood_(candidate) *= normpdf(spectrum_(partials_(i)), amp/4, 0) / normpdf(amp, amp/4, 0);
	}
	
	return likelihood_(candidate);
}


inline int
Pitch::round(float r)
{
  if(r - floor(r) <= 0.5 ) return (int)floor(r);
  return (int)ceil(r);
}

  
  


