/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/**
   \class PitchDist
   \brief FFT based PitchDist features 


   Calculate FFT mangitude based features. 
The Centroid, Rolloff and Flux are calculated 
on the magnitude spectrum of the input fvec 
and Zcrs are calculated in time domain. 
This System feature are typically used in classification. 
*/


#include "PitchDist.h"

PitchDist::PitchDist()
{
  inSize_ = DEFAULT_WIN_SIZE;
  numPitches_ = 5;
  init();
}

PitchDist::PitchDist(unsigned int inSize, unsigned int pitches)
{
  inSize_ = inSize;
  pitches_ = pitches;
  init();
}


void 
PitchDist::init()
{

  in_.create(inSize_);
   
  for (i=0; i < pitches_; i++)
    featNames_.push_back("pf");
    
  esacf_.create(inSize);
  multipitch_ = new MultiPitch(inSize);
  res_.create(pitches_);
  picker_ = new PeakPicker(inSize_, 1, pitches_, 22, inSize_, 50);
  peaks_.create(2 * pitches_);
    
}



PitchDist::~PitchDist()
{
  delete picker_;
  delete multipitch_;
}


void 
PitchDist::process(fvec& in, fvec& out) 
{
  unsigned int i;
  assert((in.size() == inSize_) && (out.size() == outSize_));      
  
  series_->process(in, temp0_);
  zcrs_->process(in, temp1_);
  rms_->process(in, temp2_);
  
  for (i=0; i<3; i++)
    {
      out(i) = temp0_(i);
    }
  out(3) = temp2_(0); 
}


	

	
