/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class Pitches
   \brief FFT based Pitches features 


   Calculate FFT mangitude based features. 
The Centroid, Rolloff and Flux are calculated 
on the magnitude spectrum of the input fvec 
and Zcrs are calculated in time domain. 
This System feature are typically used in classification. 
*/

#if !defined(__Pitches_h)
#define __Pitches_h

#include "MultiPitchExtractor.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "Centroid.h"
#include "DB.h"
#include "Series.h"
#include "RMS.h"
#include "WaveletPyramid.h"
#include "Daub4.h"
#include "FullWaveRectifier.h"
#include "OnePole.h" 
#include "Norm.h"
#include "AutoCorrelationFFT.h" 
#include "DownSampler.h" 
#include "PeakPicker.h" 
#include "NormRMS.h"
#include "MultiPitch.h"

class Pitches: public System
{
private:
  MultiPitch* multipitch_; 
  PeakPicker *picker_;  

  unsigned int pitches_;
  unsigned int inSize_;
  
  vector<string> featNames_;
  fvec esacf_;
  fvec res_;
  fvec in_;
  fvec peaks_; 
  fvec histo_;
  fvec fhisto_;
  
  
public:
  Pitches();
  Pitches(unsigned int inSize, unsigned int pitches);
  ~Pitches();
  void init();
  void process(fvec& in, fvec& out);
};

#endif

	
