/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class  PitchExtractor
   \brief  Single Vector feature extractor for monophonic pitch

   This is an example of a specific extractor for calculating 
a single feature vector. It is used for calculating the pitch 
of monophonic sound sources. 
*/


/* PitchExtractor:
   Iterate through a sound object and plot each window
*/ 

#if !defined(__PitchExtractor_h)
#define __PitchExtractor_h

#include "Extractor.h"
#include "PitchExtractor.h"
#include "ValleyPick.h"
#include "AMDF.h"
#include "Series.h"
#include "Median.h"

class PitchExtractor:public Extractor
{
protected:
  fvec in_;
  fvec out_;
  fvec res_;
  fvec pitchVec_;
  
  
  AMDF* amdf_;
  ValleyPick* vpick_;
  Series *series_;
  Median *median_;
  
  unsigned int featSize_;
  vector<string> featNames_;
  
  
public:
  PitchExtractor(MSignal* in);
  ~PitchExtractor();
  void extract();
  void extract(unsigned int start, unsigned int end);
  void extract(unsigned int start, unsigned int end, FeatMatrix& fmatrix);
  unsigned int outSize();
};


#endif	


	
