/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/** 
    \class PitchHistoFeatures
    \brief Calculate features of beat histogram

    Calculate features for beat histogram.
*/





#include "PitchHistoFeatures.h"

PitchHistoFeatures::PitchHistoFeatures()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = 5;
  numPeaks_ = 3;
  temp_.create(12);
}

PitchHistoFeatures::~PitchHistoFeatures()
{
}


PitchHistoFeatures::PitchHistoFeatures(unsigned int inSize)
{
  inSize_ = inSize;
  outSize_ = 5;
  numPeaks_ = 3;
  temp_.create(12);
}


void 
PitchHistoFeatures::process(fvec& in, fvec& out) 
{
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  unsigned int i,k,j; 
  
  float max;
  float fmax;
  
  max = 0.0; 
  fvec maximums;
  ivec maxPositions;
  fvec fmaximums;
  ivec fmaxPositions;
  
  maximums.create(numPeaks_);
  maxPositions.create(numPeaks_);
  fmaximums.create(numPeaks_);
  fmaxPositions.create(numPeaks_);
  
  int BW = 5;

  // Map to circle of fifths histogram 

   
  for (i=0; i < 12; i++)
    temp_(i) = in(i);
  
  in(0) = temp_(0);
  in(1) = temp_(7);
  in(2) = temp_(2);
  in(3) = temp_(9);
  in(4) = temp_(4);
  in(5) = temp_(11);
  in(6) = temp_(6);
  in(7) = temp_(1);
  in(8) = temp_(8);
  in(9) = temp_(3);
  in(10) = temp_(10);
  in(11) = temp_(5);
   

  
  // Folded peaks 
  for (k=0; k < numPeaks_; k++)
    {
      fmax = 0.0;
      for (i=1; i < 12; i++) 
	{
	  if ((in(i) > max) && (in(i) > in(i-1)) && (in(i) > in(i+1)))
	    {
	      fmax = in(i);
	      fmaximums(k) = fmax;
	      fmaxPositions(k) = i;
	    }
	}
      in(fmaxPositions(k)) = 0.0;

      // if (fmaxPositions(k) > BW)
      // for (j= fmaxPositions(k)-BW; j < fmaxPositions(k)+BW; j++)
      // in(j) = 0.0;
    }

  // Unfolded peaks 
  for (k=0; k < numPeaks_; k++)
    {
      max = 0.0;
      for (i=13; i < in.size()-1; i++) 
	{
	  if ((in(i) > max) && (in(i) > in(i-1)) && (in(i) > in(i+1)))
	    {
	      max = in(i);
	      maximums(k) = max;
	      maxPositions(k) = i;
	    }
	}
      in(maxPositions(k)) = 0.0;
      
      if (maxPositions(k) > BW)
	for (j= maxPositions(k)-BW; j < maxPositions(k)+BW; j++)
	  in(j) = 0.0;
    }

  
  
  /* float sum = 0.0;
  for (i=0; i < numPeaks_; i++)  
    sum += maximums(i);
  */

  // folded average
  float favg = 0.0;
  for (i=0; i < 12; i++)  
    favg += in(i);
  favg /= inSize_;

  // unfolded average
  float avg = 0.0;
  for (i=15; i < inSize_; i++)  
    avg += in(i);
  avg /= inSize_;

  

  
  
  out(0) = fmaximums(0);
  out(1) = maxPositions(0);
  out(2) = favg;
  out(3) = fmaxPositions(0);
  out(4) = fmaxPositions(0) - fmaxPositions(1);
  
}

	
