/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/**
   \class  PlotExtractor
   \brief  Extractor for waveform plots

   Iterate over a MSignal and produce a waveform plot stored in a fvec.
*/


#include "PlotExtractor.h"

PlotExtractor::PlotExtractor(MSignal* src, unsigned int lineSize):FvecExtractor(src, lineSize)
{
}

PlotExtractor::~PlotExtractor()
{
}



void 
PlotExtractor::extract(unsigned int start, unsigned int end, 
		      fvec& res) 
{
  
  unsigned int t, i;
  float max = 0.0;
  for (t=start; t< end; t++)
    {
      src_->get(in_, t);
      for (i=0; i < lineSize_; i++)
	{
	  if (fabs(in_(i)) > fabs(max))
	    max = in_(i);
	}
      res(t-start) = max;
      max = 0;
    }
}



