/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/**
   \class Sine
   \brief Sine wave Instrument
   
   Instrument for generating single Sine waves of 
a particular frequency, amplitude and duration. 
*/


#include "Sine.h"


Sine::Sine(unsigned int srate):Instrument(srate)
{
}



void 
Sine::init(float freq, float amp, float duration)
{
  freq_ = freq;
  amp_ = amp;
  duration_ = duration;
  wavetableSize_ = (long)ceil(duration_ * srate_);
  wavetable_.create(wavetableSize_);
  float incr = (TWO_PI / srate_) * freq;
  for (unsigned long i=0; i< wavetableSize_; i++)
    {
      wavetable_(i) = amp_ * sin(incr * i);
    }
}

void 
Sine::init_note(short note_num, float amp, float duration)
{
  init(pitch_table_[note_num], amp, duration);
}




void 
Sine::get(fvec& win, unsigned long index)
{
  unsigned int winSize = win.size();
  unsigned int pos = winSize * index;
  unsigned int i;
  unsigned int j;
  cerr << "Sine::get_fvec called\n" << endl;
  for (i= (pos + skipSize_) % wavetableSize_, j=0; j<winSize; i++,j++)
    {
      win(j) = wavetable_(i);
    }
}

void
Sine::put(fvec& win, unsigned long index)
{
}

unsigned long 
Sine::iterations()
{
  return wavetableSize_ / winSize_;
}
