/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class SoundFileIO
   \brief Interface for sound file IO
   
   Abstract base class for any type of sound file IO.
Specific IO classes for various formats like AuFileIO are children 
of this class. 
*/

	

#if !defined(__SoundFileIO_h)
#define __SoundFileIO_h

#include "MSignal.h"
#include <string.h>
#include "fvec.h"
#include <stdio.h>
#include <math.h>

#ifndef WIN32
#include <netinet/in.h>
#else
#include <winsock.h>
#endif

#define UnsignedToFloat(u) (((double)((long)(u - 2147483647L - 1))) + 2147483648.0)


# define FloatToUnsigned(f)      ((unsigned long)(((long)(f - 2147483648.0)) + 2147483647L) + 1)



class SoundFileIO: public MSignal
{
private:
  short *sdata_;
  unsigned char *cdata_;
  unsigned int sdataSize_;
  
  long *ldata_;
  void ConvertToIeeeExtended(double num,char *bytes);
  double ieee_ext2double(unsigned char *bytes);
protected:
  FILE *sfp_;  
  long sfp_begin_;
  string filename_;
  int mode_;  
public:
  SoundFileIO(string filename, int mode);
  ~SoundFileIO();
  virtual void get(fvec& win, unsigned long win_start) = 0;
  virtual void put(fvec& win, unsigned long win_start) = 0;
  virtual int  check_type() = 0;
  virtual void init() = 0;
  void initWindow(unsigned int winSize, unsigned int hopSize, 
			   unsigned int skipSize, unsigned int zeroSize);
  
  
  void debug_info();
  double read_ieee_extended();
  void write_ieee_extended(FILE *fp, double num);
  void read_linear16Swap(fvec& win, unsigned long index);
  void read_linear16(fvec& win, unsigned long index);
  void read_linear8(fvec& win, unsigned long index);
  void write_linear16(fvec& win, unsigned long index);
  void write_float(fvec& win, unsigned long index);
  void read_float(fvec& win, unsigned long index);
  unsigned long iterations();
  
};



#endif
