/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/



/** 
    \class TimeLine 
    \brief TimeLine is a list of TimeRegions. 
    
   TimeLine is a list of TimeRegions. It represents a segmentation in
   time of sound.  Each segment is defined by its boundaries and a
   class id. A class_name table is used to derive the class names from
   the class id's.  A simple float_vector representation is used for
   the TimeLine where the class #ids and the boundaries are
   interleaved.  For example the following numbers:

   0 1 100 0 258 3 640
   
   mean from time 0 to 100 class is 1
   from time 100 to 258 class is 0
   etc.
   
   The units used to define the boundaries of 
   each segment can be arbitrary integer multiples
   of one sample and are defined by lineSize.  

*/





#if !defined(__TimeLine_h)
#define __TimeLine_h


#include "fvec.h"
#include "Communicator.h"
#include "Color.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "math.h"
#include "FeatMatrix.h"


struct TimeRegion
{
  unsigned int start;
  unsigned int classId;
  Color color;
  unsigned int end;
};


#define SEPCHARS " \t\n"

class TimeLine
{
protected:
  TimeRegion *regions_;
  int srate_;
  vector<string> classNames_;
  int lineSize_;
  int numRegions_;
  int size_;
  string filename_;
public:
  TimeLine();
  ~TimeLine();
  TimeLine(const TimeLine& a);
  void scan(fvec segmentation);
  /* void color(FeatMatrix &map); */
  void operator=(const TimeLine& a);
  int numRegions();
  int start(int regionNum);
  int end(int regionNum);
  void init(fvec values);
  void load(string filename);
  void send(Communicator* com);
  void info();
  //void lengths_info();
  void print(FILE *fp);
  //void print_mp3(FILE *fp);
  //void print_mmf(FILE *fp);
  
};



#endif



	



	
