/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/



/** 
    \class ValleyPick
    \brief Multiply input fvec with gain

    Simple Subsample Valley Pick.
*/



#include "ValleyPick.h"

ValleyPick::ValleyPick()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = DEFAULT_WIN_SIZE;
  maxPeriod_ = inSize_;
  minPeriod_ = 10;
}

ValleyPick::ValleyPick(unsigned int inSize)
{
  inSize_ = inSize;
  outSize_ = 1;
  maxPeriod_ = inSize_;
  minPeriod_ = 10;
}

ValleyPick::~ValleyPick()
{
}



void 
ValleyPick::process(fvec& in, fvec& out) 
{
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  float avg = 0.0;
  unsigned int i;
  float valThres = 0.7 * avg;
  unsigned int period = minPeriod_;
  float delta;
  
  for (i=0; i < inSize_; i++)
    avg += in(i);
  avg /= inSize_;
  
  // Pick valley
  for (i=minPeriod_; i < maxPeriod_-1; i++)
    {
      if ((in(i) <= valThres) 
	  &&(in(i-1) > in(i))
	  &&(in(i+1) > in(i)))
	{
	  period = i;
	}
    }
  
  // Parabola fitting for subsample accuracy */ 
  float left, middle, right;
  left = in(period-1);
  middle = in(period);
  right = in(period+1);
  
  delta = (float) (left - (2 * middle) + right);
  if (delta != 0.0) delta = (float) (left-right) /delta;
  
  out(0) = period + delta;
}


	
