/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class WaveletPyramid
    \brief Pyramid wavelet algorithm

   Discrete Wavelet Transform (DWT) pyramid algorithm.
Based on the Numerical Recipies wavelet code. 
*/







#include "WaveletPyramid.h"

WaveletPyramid::WaveletPyramid()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = DEFAULT_WIN_SIZE;
  direction_ = WVL_FORWARD;
}

WaveletPyramid::WaveletPyramid(unsigned int inSize, WaveletStep* waveletStep,
			       int direction)
{
  inSize_ = inSize;
  outSize_ = inSize;
  waveletStep_ = waveletStep;
  direction_ = direction;

}


void 
WaveletPyramid::process(fvec& in, fvec& out) 
{
  unsigned int i;
  unsigned long nn;
  unsigned long n;
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  
  n = inSize_;
  if (n < 4) return;
  for (i=0; i<in.size(); i++)
    out(i) = in(i);
  
  if (direction_ == WVL_FORWARD)
    {
      for (nn= n; nn >=4; nn >>=1) 
	{
	  waveletStep_->setSize(nn);
	  waveletStep_->process(out, out);	  
	}
    }
  else if (direction_ == WVL_INVERSE)
    {
      for (nn=4; nn <= n; nn <<=1)
	{
	  waveletStep_->setSize(nn);
	  waveletStep_->process(out, out);
	}
    }
  
  
}


	
