/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include "FeatMatrix.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "MagDB.h"
#include "Sine.h"
#include "Spectral.h"
#include "LabelList.h"
#include "Collection.h"
#include "Gaussian.h"
#include "K_MEANS.h"

#ifndef WIN32
#include "Audio.h"
#else 
#include "system/AudioDummy.h"
#endif

void 
cluster(string clustererstr, string collectionstr, string ocollectionstr, 
	unsigned int clusters)
{
  cerr << "Clustering " << collectionstr << " using " << clustererstr << " and " 
       << clusters << " clusters." << endl;
  if (clustererstr != "K_MEANS")
    {
      cerr << "Unsupported clusterer\n" << endl;
      return;
    }
  K_MEANS kmeans;
  kmeans.init(clusters);
  kmeans.clusterCollection(collectionstr, ocollectionstr);
}

 

int
main(int argc, char **argv)
{
  if (argc != 5)
    {
      cerr << "Usage :" << argv[0] <<" Clusterer featurecollection originalcollection #clusters\n";
      cerr << "For now only K_MEANS is a valid clusterer\n" << endl;
      exit(1);
    }
  string clustererstr = argv[1];
  unsigned int clusters = atoi(argv[4]);
  
  string collectionstr = MARSYAS_MFDIR;
  collectionstr += "/collections/";
  collectionstr += argv[2];
  
  string ocollectionstr = MARSYAS_MFDIR;
  ocollectionstr += "/collections/";
  ocollectionstr += argv[3];

  cluster(clustererstr, collectionstr, ocollectionstr,clusters);
  
}
