/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>

#include "FeatMatrix.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "defs.h"
#include "Collection.h"
#include "ColorPlot.h"
#include "FileName.h"

void
colorgram(string srcStr, FeatMatrix& src, fvec stats)
{
  unsigned int i;
  
  fvec pc1(src.rows());
  fvec pc2(src.rows());
  fvec pc3(src.rows());

  
  pc1 = src.col(8);
  pc2 = src.col(7);
  pc3 = src.col(6);

   
  pc1.renorm(stats(0), stats(1), 0.5, 0.3);
  pc2.renorm(stats(2), stats(3), 0.5, 0.3);
  pc3.renorm(stats(4), stats(5), 0.5, 0.3);
   
  
  /* unsigned int skip = 40;
  
  if (src.rows() > 40)
    skip = 40;
  else
    skip = 0;
  */
  unsigned int skip = 0;
  
  fvec rvec(src.rows()-skip);
  fvec gvec(src.rows()-skip);
  fvec bvec(src.rows()-skip);
  
  for (i=0; i < rvec.size(); i++)
    {
      rvec(i) = pc1(i+skip);
      gvec(i) = pc2(i+skip);
      bvec(i) = pc3(i+skip);
    }

  
  ColorPlot cplot;
  cplot.init(src.name(), rvec, gvec, bvec);
  FileName fname(src.name());
  cout << "name = " << fname.name() << endl;
  string destStr;
  destStr += MARSYAS_MFDIR;
  destStr += "/timbregrams/";
  destStr += fname.name();
  destStr += ".mcl";
  cplot.write(destStr);
  rvec.write(destStr + "r");
  gvec.write(destStr + "g");
  bvec.write(destStr + "b");
  cout << "Writing " << destStr << endl;
}

 

int
main(int argc, char **argv)
{
  unsigned int cl,m,i;
  
  if (argc == 1)
    {
      cerr << "Usage :" << argv[0] <<" collections \n";
      exit(1);
    }
  unsigned int numCollections = argc -1;
 
  for (cl =0; cl < numCollections; cl++)
    {
      cout << "Reading collection " << cl << " " << argv[cl+1] << endl;
    }
  vector <FeatMatrix *> allMatrices;
  allMatrices.reserve(1000);
  vector <string> allNames;
  
  unsigned int count = 0;
  unsigned int size;
  
  for (cl = 0; cl < numCollections; cl++)
    {
      string collectionstr = MARSYAS_MFDIR;
      collectionstr += "/featurefiles/";
      collectionstr += argv[cl+1];
      ifstream from(collectionstr.c_str());
      cout << "Reading collection " << cl << " " << collectionstr << endl;
      allMatrices.push_back(new FeatMatrix());
      while (from >> *(allMatrices[count]))
	{
	  allMatrices.push_back(new FeatMatrix());
	  allNames.push_back(allMatrices[count]->name());
	  cout << "reading " << allMatrices[count]->name() << endl;
	  count++;
	}
      allMatrices.pop_back();
    }


  FeatMatrix trainMatrix(allMatrices);
  
  
  /* Collection destCollection;
  string destName = (l.name() + "prj");
  destCollection.setName(destName);
  
  
  string fdir = "../mf/collections/";
  */ 
  string destEntry;  
  
  fvec stats(6);
  fvec pc1(trainMatrix.rows());
  fvec pc2(trainMatrix.rows());
  fvec pc3(trainMatrix.rows());
  pc1 = trainMatrix.col(8);
  pc2 = trainMatrix.col(7);
  pc3 = trainMatrix.col(6);
  
  
  stats(0) = pc1.mean();
  stats(1) = pc1.std();
  stats(2) = pc2.mean();
  stats(3) = pc2.std();
  stats(4) = pc3.mean();
  stats(5) = pc3.std();
  
  /* string projectstr = MARSYAS_MFDIR;
  projectstr += "/projectfiles/";
  projectstr = "MusicSpeechFFTsta.mf";
  
  stats.write(projectstr);
  */
  cout << "Do I get here " << endl;

  for (i=0; i < count; i++)
    {
    colorgram(allNames[i], *(allMatrices[i]), stats);
    }
    
    //destCollection.add(destEntry);
  //}
  // destCollection.write(fdir + destName + ".mf");
  // cout << "Wrote " << (fdir + destName + ".mf") << endl;
  return 0;
  
}
