/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* similar:
   Command line utility for finding content-based similarity 
retrieval.
*/

#if HAVE_CONFIG_H
#include <config.h>
#endif


#include <stdio.h>
#include "FeatMatrix.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "MagDB.h"
#include "Sine.h"
#include "Spectral.h"
#include "SpectralSegm.h"
#include "SfxExtractor.h"
#include "MemMFCC.h"
#include "LPC.h"
#include "MultiPitchFeature.h"
#include "DWTC.h"
#include "BeatExtractor.h"
#include "SvFeatExtractor.h"
#include "MultiPitchExtractor.h"
#include "GenreExtractor.h"
#include "ExtractorDictionary.h"
#include "FileName.h"
#include "KNN.h"
#include "Gaussian.h"
#include "Classifier.h"


#include <map>
#include <string.h>
using namespace std;

#include "MSignal.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"

#ifndef WIN32
#include "Audio.h"
#else
#include "system/AudioDummy.h"
#endif


void 
play(float seconds, string filename, AuFileIO *dest)
{
  MSignal *src;
  Util util;
  src = util.sfopen(filename, MRS_SF_READ);
  if (src == NULL) 
    exit(1);
  src->initWindow(1024, 1024, 0, 0);
  fvec window(src->winSize());
  // Audio audio(src->srate(),1, MRS_AUDIO_PLAY, src->winSize());
  float iterations = (seconds * src->srate()) / src->winSize();
  if ((iterations >= src->iterations())||(seconds == -1.0))
    iterations = src->iterations();
  for (unsigned int i=0; i< iterations; i++)
    {
      src->get(window,i);
      // audio.put(window,i);
      // dest->put(window,i);
    }
  delete src;
}



void 
estimate(string classifierStr, string extractorStr, string audiofileStr)
{
  MSignal *src;
  Util util;
  src = util.sfopen(audiofileStr, MRS_SF_READ);
  src->initWindow(512, 512, 0, 0);
  
  /* Special window sizes for specific extractors */ 
  if (extractorStr == "LPC")
    src->initWindow(512, 128, 0, 0);
  if (extractorStr == "BEAT") 
    src->initWindow(65536, 65536/2,0,0);
  if (extractorStr == "DWTC")
    src->initWindow(16384, 512, 0, 0);
  if (extractorStr == "MPITCH")
    src->initWindow(512, 512, 0, 0);
  
  /* Extractor dictionary contains all the available 
     extractors for feature calculation */ 
  ExtractorDictionary dict(src);
  
  Extractor* extr = dict.getExtractor(extractorStr);
  
  /* find appropriate extractor and feature matrix result */ 
  unsigned int rows = dict.getRows(extractorStr);
  unsigned int cols = dict.getCols(extractorStr);
  /* query feature matrix */
  FeatMatrix* qfeatMatrix = new FeatMatrix(rows, cols);
  
  extr->extract(*qfeatMatrix);
  
  Gaussian classifier;
  ifstream classifierFrom(classifierStr.c_str());
  classifierFrom >> classifier;
  
  unsigned int label = classifier.estimateClass(*qfeatMatrix);
  string strlabel = classifier.estimate(*qfeatMatrix);
  cout << "<br> Estimated GENRE = " << strlabel << endl;
  cout << "<br> Supported Genres = Classical, Country, Disco, HipHop, Jazz, Rock, Blues, Reggae, Pop, Metal" << endl;
  cout << "<br>" << endl;
  
  
}



void similar(string extractorStr, string audiofileStr, int k, 
	     string collectionStr, string linkStr, string classifierStr)
{
  unsigned int i;
  /* Prepare name (remove full path) for feature matrix */ 
  FileName fromFile(audiofileStr);
  string fromName = fromFile.name();  
  string mf("mf");
  

  FeatMatrix* qfeatMatrix = new FeatMatrix();
  
  if (fromFile.ext() == mf)		// File is a feature matrix
    {
      cout << "reading from " << audiofileStr << endl;
      qfeatMatrix->read(audiofileStr);
    }
  else					// Audio file extract features
    {
      MSignal *src;
      Util util;
      src = util.sfopen(audiofileStr, MRS_SF_READ);
      src->initWindow(512, 512, 0, 0);
      
      /* Special window sizes for specific extractors */ 
      if (extractorStr == "LPC")
	src->initWindow(512, 128, 0, 0);
      if (extractorStr == "BEAT") 
	src->initWindow(65536, 65536/2,0,0);
      if (extractorStr == "DWTC")
	src->initWindow(16384, 512, 0, 0);
      if (extractorStr == "MPITCH")
	src->initWindow(512, 512, 0, 0);
      
      /* Extractor dictionary contains all the available 
	 extractors for feature calculation */ 
      ExtractorDictionary dict(src);
      
      Extractor* extr = dict.getExtractor(extractorStr);
      
      
      /* find appropriate extractor and feature matrix result */ 
      delete qfeatMatrix;
      unsigned int rows = dict.getRows(extractorStr);
      unsigned int cols = dict.getCols(extractorStr);
      /* query feature matrix */
      qfeatMatrix = new FeatMatrix(rows, cols);
      
      extr->extract(*qfeatMatrix);
      qfeatMatrix->setName(fromName);
    }
  Gaussian gclassifier;
  ifstream classifierFrom(classifierStr.c_str());
  classifierFrom >> gclassifier;
  
  unsigned int label = gclassifier.estimateClass(*qfeatMatrix);
  string strlabel = gclassifier.estimate(*qfeatMatrix);
  cout << "<br> Estimated GENRE = " << strlabel << endl;
  cout << "<br> Supported Genres = Classical, Country, Disco, HipHop, Jazz, Rock, Blues, Reggae, Pop, Metal" << endl;
  cout << "<br>" << endl;
  
  
      
  /* Read the training collection */
  FeatMatrix trainMatrix;
  trainMatrix.read(collectionStr);
  
  KNN classifier(1);
  // Gaussian classifier;
  
  classifier.train(&trainMatrix,1);

  // Return the rank list 
  vector<string> rlist;
  
  rlist = classifier.rankList(*qfeatMatrix, k);

  cout << "<br> QUERY: " << endl;
  string mp3Str = "./tmp/query.mp3";
  string dStr = audiofileStr.substr(0, audiofileStr.size()-3);
  
  cout << "<br> <a href=\"" << linkStr << "\"> " << dStr << "</a>"    << endl;  
  // play(10, audiofileStr, dest1);
   cout << "<br> SIMILAR: " << endl; 
   for (i=1; i < 4; i++)
     {
       cout << "<br> <a href=\"" << rlist[i] << "\"> " << rlist[i] << "</a>"    << endl;
       // play(10, rlist[i], dest1);
     }
}



	
int 
main(int argc, char **argv)
{
  if (argc == 1)
    {
      cerr << "Usage :" << argv[0] << " extractor audiofile k collection linkStr classifier" << endl;
      exit(1);
    }
  string extractorStr = argv[1];
  string audiofileStr = argv[2];
  int k = atoi(argv[3]);
  string cStr = argv[4];
  string linkStr = argv[5];
  string classifierStr = argv[6];
  
  
  
  string collectionStr = cStr;
  collectionStr += extractorStr;

  // estimate(classifierStr, extractorStr,audiofileStr);  
  similar(extractorStr, audiofileStr, k, collectionStr, linkStr, classifierStr);
  
  return 0;
}
