/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include "FeatMatrix.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "MagDB.h"
#include "Sine.h"
#include "Spectral.h"
#include "LabelList.h"
#include "Collection.h"
#include "Gaussian.h"
#include "ExtractorDictionary.h"

void 
estimate(string classifierStr, string extractorStr, string audiofileStr)
{
  MSignal *src;
  Util util;
  src = util.sfopen(audiofileStr, MRS_SF_READ);
  src->initWindow(512, 512, 0, 0);
  
  /* Special window sizes for specific extractors */ 
  if (extractorStr == "LPC")
    src->initWindow(512, 128, 0, 0);
  if (extractorStr == "BEAT") 
    src->initWindow(65536, 65536/2,0,0);
  if (extractorStr == "DWTC")
    src->initWindow(16384, 512, 0, 0);
  if (extractorStr == "MPITCH")
    src->initWindow(512, 512, 0, 0);
  
  /* Extractor dictionary contains all the available 
     extractors for feature calculation */ 
  ExtractorDictionary dict(src);
  
  Extractor* extr = dict.getExtractor(extractorStr);
  
  /* find appropriate extractor and feature matrix result */ 
  unsigned int rows = dict.getRows(extractorStr);
  unsigned int cols = dict.getCols(extractorStr);
  /* query feature matrix */
  FeatMatrix* qfeatMatrix = new FeatMatrix(rows, cols);
  
  cout << "Extracting features " << endl;
  extr->extract(*qfeatMatrix);
  
  Gaussian classifier;
  ifstream classifierFrom(classifierStr.c_str());
  classifierFrom >> classifier;
  
  unsigned int label = classifier.estimateClass(*qfeatMatrix);
  string strlabel = classifier.estimate(*qfeatMatrix);
  cout << "Estimated class = " << label << "(" << strlabel << ")" << endl;
}

 

int
main(int argc, char **argv)
{
  if (argc != 4)
    {
      cerr << "Usage :" << argv[0] <<" Classifier Extractor AudioFile\n";
      exit(1);
    }
  string classifyStr = MARSYAS_MFDIR;
  classifyStr += "/classifiers/";
  classifyStr += argv[1];
  string extractStr = argv[2];
  string audioStr = argv[3];
  estimate(classifyStr, extractStr,audioStr);
  return 0;
}
