/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/



#include "FeatMatrix.h"
#include "Collection.h"
#include "defs.h"
#include "Gaussian.h"
#include "GMM.h" 
#include "KNN.h" 
#include "ClassifierDictionary.h"
#include "ExtractorDictionary.h" 

#include <stdio.h>
#include <vector> 
using namespace std;




fmatrix totalConfusion_;


float
eval(string classifierName, FeatMatrix* trainMatrix, 
     FeatMatrix* evalMatrix, unsigned int count)
{
  
  ClassifierDictionary cdict;
  Classifier* classifier = cdict.getClassifier(classifierName);
  float error = 1.0;


  classifier->train(trainMatrix,0);      
  
  error = classifier->evaluate(evalMatrix);
  cout << "(" << count << ") Classification Error = " << error * 100.0 << "%" << endl;
  cout << "(i.e Correct Classifications = " << (1.0 - error) * 100.0 << "%)" << endl;  

  fmatrix confusion;
  confusion.create(classifier->classSize(), classifier->classSize());
  classifier->confusion(evalMatrix, confusion);  
  
  totalConfusion_ += confusion;
  return error;
}





int 
main(int argc, char **argv) 
{
  ClassifierDictionary cdict;
  ExtractorDictionary edict;
  
  if (argc == 1) 
    {
      cerr << "Usage: " << argv[0] << " seed classifier train iterations" << endl;
      // cerr << "Supported extractors: " << endl;
      // cerr << edict << endl;
      cerr << "Supported classfiers: " << endl;
      cerr << cdict << endl;
      exit(1);
    }
  // unsigned int numClasses = argc - 4;
  
  unsigned int i,cl;
  
  string classifierStr;
  string trainStr = MARSYAS_MFDIR;
  trainStr += "/featurefiles/";
  
  
  int seed = atoi(argv[1]);
  srand(seed);  
  classifierStr = argv[2];
  trainStr += argv[3];
  int iterations = atoi(argv[4]);


  cout << "Seed = " << seed << endl;
  cout << "Classifier = " << classifierStr << endl;
  cout << "Training set = " << trainStr << endl;
  cout << "Iterations = " << iterations << endl;

  FeatMatrix full;
  cout << "Started reading: " << trainStr << endl;
  full.read(trainStr);
  cout << "Finished reading: " << trainStr << endl;
  
  
  float totalError = 0.0;

  fvec results(iterations);  
  ivec partition(full.rows());
  
  unsigned int rind;
  
  srand(seed);
  unsigned int m,r;
  

  
  for (i=0; i < iterations; i++)
    {
      unsigned int testCount = 0;
      partition.setval(0);
      
      while (testCount < 0.1 * full.rows())
	{
	  rind = (unsigned int)(((float)(rand() /(float)(RAND_MAX)))* full.rows());
	  for (r=0; r < full.rows(); r++)
	    {
	      if ((full.labelNum(r,0) == full.labelNum(rind,0))
		  &&(full.labelNum(r,1) == full.labelNum(rind,1))
		  &&(partition(r) == 0))
		{
		  partition(r) = 1;
		  testCount++;
		}
	    }
	}
      FeatMatrix trainMatrix(full, partition, 0);
      FeatMatrix evalMatrix(full, partition, 1);

      if (i==0) 
	totalConfusion_.create(trainMatrix.labelSize(0), trainMatrix.labelSize(0));
      
      results(i) = eval(classifierStr, &trainMatrix, &evalMatrix, i);      
    }
  
  totalConfusion_ /= iterations;
  totalConfusion_ *= 100.0;
  totalError = results.mean();
  float stdError = results.std();
  unsigned int numClasses = full.labelSize(0);
  
  
  cout << "\n(Theoretical Random Classifier Correct Classifications = " << 100.0 / numClasses << ")" << endl;
  cout << "Total Classification Error = " << totalError*100.0 <<"%" << endl;
  cout << "(i.e Total Correct Classifications = " << (1.0 -totalError)*100.0 
       << "+/-" << stdError * 100.0 << "%" << endl;    
  
  cout << "Final confusion matrix = " << totalConfusion_ << endl;
  return 0;
}




