#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "FeatMatrix.h"
#include "BatchExtract.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "MagDB.h"
#include "Sine.h"
#include "Spectral.h"
#include "SpectralSegm.h"
#include "FeatExtractor.h"
#include "Shuffler.h" 
#include "MFCC.h"
#include "ExtractorDictionary.h"
#include "Classifier.h"
#include "MemMFCC.h"
#include "LPC.h"
#include "SfxExtractor.h"
#include "DWTC.h"
#include "BeatExtractor.h"
#include "SvFeatExtractor.h"
#include "MultiPitchExtractor.h"
#include "GenreExtractor.h"
#include "MultiPitchFeature.h"
#include "Parallel.h"
#include "Series.h"
#include "ValleyPick.h"
#include "AMDF.h"
#include "knnTree.h"
#include "FFT.h"
#include "AutoCorrelation.h"
#include "Pitch.h"
#include "SpectralPeak.h"
#include "Messager.h"
#include "SKINI.h"
#include "SKINI.msg"
#include <map>
#include <vector>

#include "RtDuplex.h"
#include "RtWvIn.h"
#include "RtWvOut.h"
#include "WvOut.h"
#include "WvIn.h"
#include "waveio.h"



AuFileIO *input;
AuFileIO *fileoutput;
int cursample;
RtWvOut *output;
RtDuplex *inout;
fvec s;
struct soundhdr hdr;
FILE* outfile;
int totalsamples = 0;
int winSize_;


inline float tick(float out)
{
  short data = out * 32000;
  totalsamples++;
	fwrite(&data, 2, 1, outfile);
  if(inout != NULL) return inout->tick(out);
	output->tick(out);
	if(cursample%winSize_ == 0) input->get(s, cursample/winSize_);
	return s((cursample++)%winSize_);
}



void sort(float *a, int *b, int l, int r)
{
  static float m;
  static int j;
  int i;

  if(r > l) {
    m = a[b[r]]; i = l-1; j = r;
    for(;;) {
      while(a[b[++i]] < m) if(i >= j) break;
      while(a[b[--j]] > m) if(i >= j) break;
      if(i >= j) break;
      std::swap(b[i], b[j]);
    }
    std::swap(b[i], b[r]);
    sort(a, b, l, i-1);
    sort(a, b, i+1, r);
  }
}


inline int round(float r)
{
  if(r - floor(r) <= 0.5 ) return (int)floor(r);
  return (int)ceil(r);
}

int
main(int argc, char *argv[])
{

  //parse input
  unsigned int i, j, k, dim=0, winSize = atoi(argv[1]);
  unsigned int hopSize = winSize, nExtractors = 2, featSizes[nExtractors];
  string winSizeStr = argv[1], collectName = argv[2];
  string extractors[nExtractors], nTreeFile = collectName, filename;
  float extractorWts[nExtractors];
  
	s.create(winSize);
	winSize_ = winSize;
	
	if(argc == 4) filename = argv[3];
	else filename = "empty";

  extractors[0] = "DFCC";
  extractorWts[0] = 1;
	extractors[1] = "SFFT";
	extractorWts[1] = 1;
  
	nTreeFile += winSizeStr;
  nTreeFile += "-";
	nTreeFile += extractors[0];
  //nTreeFile += "-";
	//nTreeFile += extractors[1];
	
	
  //extract features for each extractor
  int *startnum = NULL;
  BatchExtract *collectExtract;
  collectExtract = new BatchExtract(collectName, winSizeStr, winSizeStr);
  FeatMatrix *collectReses[nExtractors], *collectRes;
  FeatMatrix *pitchRes;
	
  for(i = 0; i < nExtractors; i++)
  {
    collectReses[i] = collectExtract->extract(extractors[i], &startnum);
    featSizes[i] = collectReses[i]->cols();
    dim += featSizes[i];
  }
	
  //combine Featmatrixes for multiple features into collectRes and templRes
  int labelIndexes[collectReses[0]->rows()];
  fvec collectVec(collectReses[0]->rows()), *features[nExtractors];
  for(i = 0; i < collectReses[0]->rows(); i++) labelIndexes[i] = collectReses[0]->labelNum(i, 1);
  collectRes = new FeatMatrix(collectReses[0]->rows(), dim);
  collectRes->setLabelNames(collectReses[0]->labelNames());
  for(i = 0, j = 0; i < nExtractors; i++)
  {
    features[i] = new fvec(featSizes[i]);
    for(k = 0; k < featSizes[i]; k++, j++)
    {
      collectVec = collectReses[i]->col(k);
      collectRes->col(j, collectVec);
    }
  }
  
  //multiply features by weights
  fvec covariance(dim), vars(collectRes->varRow()), means(dim);
  for(i = 0, j = 0; i < nExtractors; i++) for(k = 0; k < featSizes[i]; k++, j++)
  {  
    if(vars(j) <= 0.000001) covariance(j) = 1.0;
    else covariance(j) = sqrt(1.0/vars(j));
    collectVec = collectRes->col(j);
    collectVec *= (extractorWts[i]); //* covariance(j));
    collectRes->col(j, collectVec);
  }
  
	
  //build search trees or read in if they already exists
  int n_pts=collectRes->rows();
  knnTree *tree;
  string numberstring, treeFile;
  struct stat stbuf;
  
  tree = new knnTree(collectRes, n_pts, dim, 7);
  treeFile = MARSYAS_MFDIR;
	treeFile += "/trees/";
	treeFile += nTreeFile + ".mtf";
  if(stat(treeFile.c_str(), &stbuf))
  {
    cout << "Building search tree..." << endl;
    tree->makeTree();
    cout << "Writing search tree..." << endl;
    tree->write(treeFile);
  }
  else
  {
    cout << "Reading search tree..." << endl;
    //tree->read(treeFile);
  }
  cout << "Tree creation complete." << endl;

  //sort collection by pitch
	int rows = collectRes->rows(), sorted[rows];
	collectVec = collectRes->col(0);

	float *dat = collectVec.getData();
	for(i = 0; i < rows; i++) sorted[i] = i;
  sort(dat, sorted, 0, rows-1);

  //Open up library audio files
  Util util; 
	vector<string> labels = collectRes->labelNames();
  AuFileIO *sample[labels.size()-1];
	for(i = 0; i < labels.size()-1; i++)
	{ 
	  sample[i] = (AuFileIO*)util.sfopen(labels[i+1], MRS_SF_READ);
    sample[i]->initWindow(winSize, hopSize, 0, 0);
	}
	
	fillheader(&hdr, 44100);
  outfile = opensoundout("crap.snd", &hdr);
  

  //Open realtime duplex device.
  Messager *messager;
  Stk::setSampleRate(44100.0);
  try
  {
	  inout = NULL;
		output = NULL;
	  if(filename == "empty") inout = new RtDuplex(2, Stk::sampleRate(), 0, RT_BUFFER_SIZE, 8);
		else
		{
			input = (AuFileIO*)util.sfopen(filename.c_str(), MRS_SF_READ);
			input->initWindow(winSize, winSize, 0, 0);
		  cursample = 0;
			output = new RtWvOut(2, Stk::sampleRate(), 0, RT_BUFFER_SIZE, 8);
		}
    messager = new Messager(STK_PIPE);  
  }
  catch (StkError &)
  {
    return 0;
  }

  fvec inwin, sendwindow(winSize), outwin(winSize), ovlap(winSize);
  fvec irms(1), orms(1), q(dim), pitis(2);
  int knns = 1000, diff, nns[knns], done = FALSE, type, byte2, byte3, state = 0, label;
	unsigned int otSz, ovSz = 0;
  float dist[knns], fctr, ovfSz = 0, temp, *data;
	float otfSz, pitch = 0;
  RMS rms(winSize);
	int cs = 30;
  DFCC *dfcc = new DFCC(winSize, 40, 30);
	fvec feats(32);
  inwin.create(winSize);
	MagFFT *fft_;
	fft_ = new MagFFT(winSize);
  ofstream pitches("pitches"), env("env");


	cout << "go!" << endl;
/*  
  ovlap.setval(0.0);
	fvec p(40);

	AuFileIO *test = (AuFileIO*)util.sfopen(filename.c_str(), MRS_SF_READ);
  test->initWindow(winSize, winSize, 0, 0);
	DFCC d(winSize, 40, 30);
	
	for(i = 0; i < 450; i++)
{
	test->get(inwin, i);
	d.process(inwin, p);
}
		
exit(0);
}
*/
  nns[0] = 0;
  for(i = 0; i < winSize; i++) inwin(i) = tick(0);
  float mindif, lastpitch = 0, lastpitch2 = 0;
	int closest;
	Hamming ham(winSize, 0);


  int totalwins = input->iterations(), currentwin = 0;
  //runtime loop
  while(currentwin < totalwins)
  {    
	  currentwin++;
    //process input and get window from library
		sendwindow = inwin;
		dfcc->process(sendwindow, feats);
		//sfft->process(inwin, sses);
    pitch = feats(0);
		
		//pitches << pitch << endl;
    //tree->findKnn(q, knns, nns, dist);
	  if(pitch == 0) pitch = lastpitch;
		else lastpitch = pitch;
	
		int low = 0, size = rows-1, mid = 0;
		while(low<=size && ((*collectRes)(sorted[mid], 0) < pitch*.995 || (*collectRes)(sorted[mid], 0) > pitch*1.005))
		{
			mid=((low+size)/2);
		  if((*collectRes)(sorted[mid], 0) < pitch*.995) low=mid+1;
			else if((*collectRes)(sorted[mid], 0) > pitch*1.005) size=mid-1;
		}

	  float distance, tdistance, diff;
		for(i = 2, distance = 0, nns[0] = sorted[mid]; i < 15; i++)
		{
		  diff = feats(i) - (*collectRes)(sorted[mid], i);
		  distance += diff*diff;
		}
		if(distance == 0) distance = 10000;
		for(j = mid+1; j < rows && (*collectRes)(sorted[j], 0) < pitch*1.005; j++)
			if((*collectRes)(sorted[j], 1) < .5 && (*collectRes)(sorted[j], 35) > .02 && (*collectRes)(sorted[j], 34) < 400)
		  {
		    for(i = 2, tdistance = 0; i < 15 && tdistance < distance; i++)
		    {
		      diff = feats(i) - (*collectRes)(sorted[j], i);
		      tdistance += diff*diff;
		    }
			  if(tdistance < distance && tdistance != 0)
			  {
			    distance = tdistance;
			  	nns[0] = sorted[j];
			  }
		  }
		if(mid > 0) for(j = mid-1; j >= 0 && (*collectRes)(sorted[j], 0) > pitch*.995; j--)
			if((*collectRes)(sorted[j], 1) < .5 && (*collectRes)(sorted[j], 35) > .02 && (*collectRes)(sorted[j], 34) < 400)
		  {
		    for(i = 2, tdistance = 0; i < 15 && tdistance < distance; i++)
		    {
		      diff = feats(i) - (*collectRes)(sorted[j], i);
		      tdistance += diff*diff;
		    }
			  if(tdistance < distance && tdistance != 0)
			  {
			    distance = tdistance;
				  nns[0] = sorted[j];
			  }
		  }
		cout << sorted[mid] << " " << nns[0] << endl;
		
		label = labelIndexes[nns[0]]; 
    sample[label]->get(outwin, nns[0]-startnum[label]);
		
    data = outwin.getData();
    fft_->rfft(data, winSize/2, FFT_FORWARD);
    for(i = 0; i < winSize/2; i++)
    { 
      outwin(2*i) = sqrt(outwin(2*i)*outwin(2*i)+outwin(2*i+1)*outwin(2*i+1));
      outwin(2*i+1) = 0;
    }
    fft_->rfft(data, winSize/2, FFT_INVERSE);
    
    //prepare and send output, get input


    rms.process(inwin, irms);
    rms.process(outwin, orms);
    fctr = irms(0) / orms(0);
    otfSz = 0;
    if(pitch == 0) pitch = 100;
		else pitch = winSize/pitch;
		if(pitch >= winSize/2) pitch = 100;
		otfSz = 0;
		while(otfSz < winSize/2) otfSz += pitch;
		otSz = round(otfSz);
    for(j = 0; j < winSize - otSz; j++)
		  inwin(j) = inwin(j + otSz);
    for(j = 0; j < ovSz; j++)
      inwin(winSize-otSz+j) = tick(((float)j/ovfSz) * fctr * outwin(j) + ovlap(j) * (ovfSz-j)/ovfSz);
    for(j = ovSz; j < otSz; j++)
		  inwin(winSize-otSz+j) = tick(fctr * outwin(j));
    for(j = 0, ovfSz = pitch*floor(winSize/pitch) - otfSz, ovSz = round(ovfSz); j < ovSz; j++)
		  ovlap(j) = outwin(otSz+j) * fctr; 


/*    rms.process(inwin, irms);
    rms.process(outwin, orms);
    fctr = irms(0) / orms(0);
    otfSz = 0;
    if(pitch == 0) pitch = 100;
		else pitch = (float)winSize/pitch;
		if(pitch >= winSize/2) pitch = 100;
		otfSz = 0;
		while(otfSz < winSize/2) otfSz += pitch;
		otSz = round(otfSz);
		
		int m = 0;
		float en, maxen = 0;
		for(i = 0; i < lastpitch; i++)
		{
		  for(j = 0, en = 0; j < ovSz; j++) en += outwin(j)*fctr*ovlap(j+i);
		  if(en >= maxen)
			{
			  maxen = en;
				m = i;
		  }
		}
		cout << m << endl;
		
    for(j = 0; j < winSize - otSz - m; j++) inwin(j) = inwin(j + otSz);
		for(j = 0; j < m; j++) inwin(winSize-otSz-m+j) = tick(ovlap(j));
    for(j = 0; j < ovSz; j++)
      inwin(winSize-otSz+j) = tick(((float)j/ovfSz)*fctr*outwin(j) + ovlap(j+m)*(ovfSz-j)/ovfSz);
    for(j = ovSz; j < otSz; j++) inwin(winSize-otSz+j) = tick(fctr * outwin(j));
    for(j = 0, ovfSz = pitch*floor((winSize)/pitch)-otfSz, ovSz = round(ovfSz-pitch); j < round(ovfSz); j++)
		  ovlap(j) = outwin(otSz+j) * fctr; 
	
		ham.process(outwin, outwin);
		outwin*=fctr;
	  for(i = 0; i < winSize/2; i++) inwin(i) = inwin(i+winSize/2);
		for(i = 0; i < winSize/2; i++) inwin(winSize/2+i) = tick(outwin(i)+ovlap(i));
		for(i = 0; i < winSize/2; i++) ovlap(i) = outwin(winSize/2+i);
	*/
  }
	closesoundout(outfile, totalsamples);

  return 0;
}





  //get initial samples
/*
	ofstream pitches("pitches"), partials("partials"), amps("amps"), spectrum("spectrum");
	Pitch *pit, *pit2;
	pit = new Pitch("test", winSize);
	
int n = 0;
while(n < 100)
{
 n++;
  for(i=0; i<winSize; i++) inwin(i) = inout->tick(0);
	pit->process(inwin, p);
	pitches << p(0) << endl;
}


	AuFileIO *test = (AuFileIO*)util.sfopen(collectName, MRS_SF_READ);
  test->initWindow(winSize, winSize/2, 0, 0);
	int n = test->iterations();
	pit = new Pitch(winSize);
	
  for(i = 0; i < n; i++) 
  {
    test->get(inwin, i);
    pit->process(inwin, p);
		pitches << p(0) << endl;
  }
		
  exit(0);

*/




/*	Util util;
	AuFileIO *test = (AuFileIO*)util.sfopen(collectName, MRS_SF_READ);
  test->initWindow(winSize, winSize, 0, 0);

	int num = test->iterations();
	for(i = 0; i < num; i++)
	{
	  test->get(inwin, i);
	  systems[0]->process(inwin, p);
		pitches << p(0) << endl;
	}
	 
		 npartials = model.getPartials(inwin, spectrum, quality, partials);
	  fvec ps(npartials*2), ccs(12);
	  for(i = 0; i < npartials; i++)
	  {
	    ps(i) = 2*PI*partials(i)/winSize;
	    ps(i+npartials) = spectrum(partials(i));
  	}
	  dfcc.set(npartials*2, 12);
	  dfcc.process(ps, ccs);
    
 
	  for(i=0; i<winSize/2; i++)
	  {
		  ofpar << i << " " << spectrum(i) << endl;
	    envelope << i << " " << dfcc.get(ccs, 2*PI*i/winSize) << endl;
		}
		ofpar << endl;
		envelope << endl;
	}

  float *data = in.getData(), estim;
	int j;
	magfft_->rfft(data, inSize_/2, FFT_FORWARD);
	for(i = 0; i < inSize_/2; i++)
	{ 
		data[2*i] = 20*log10(sqrt(data[2*i]*data[2*i]+data[2*i+1]*data[2*i+1]));
		logpar << data[2*i] << endl;
		data[2*i+1] = 0;
	}
	magfft_->rfft(data, inSize_/2, FFT_INVERSE);
	int order = 40;
	for(i=0; i<inSize_/2; i++)
	{
	  cepstrum << data[i] << endl;
	  estim = 0;
	  for(j = 0; j <= order; j++) estim += data[j]*cos(j*i*2*PI/inSize_);
	  envelope << estim/2048 << endl;
	}
*/


  /*for(i = 0; i < test->iterations(); i++) 
  {
    test->get(inwin, i);
    systems[0]->process(inwin, *features[0]);
    cout << (*features[0])(0) << endl;
  }
  exit(0);*/


