#include "knnTree.h"


node::node(
	int 	num,
	int		*i,   	
	node	**child,
   	fvec	princ,
   	fvec	mn,
   	fvec	mx)
{
	cc       = num;
	children = child;

	if(children != NULL) {
		principal.create(princ.size());
		principal = princ;
		min.create(mn.size());
		min = mn;
		max.create(mx.size());
		max = mx;
	}
	else index = i;
}



node::~node()
{
//if(children != NULL) for(int i = 0; i < cc; i++) delete children[i];
}



knnTree::~knnTree()
{
    //if (root != NULL) delete root;
}



void getPrincipal(fmatrix *matrix, int n, int dd, fvec& principal)
{
	fvec mean(dd);
	fvec row(dd);
	float var    = 0;
	float varold = 0;
	int i, j, nl = 1;

	float centered[n];
	mean = matrix->meanRow();
	principal.setval( 1.0 / sqrt((float)dd) );
	
	for(i = 0; i < n; i++) 
	{
		row = matrix->row(i);
		centered[i]=0;
		for(j = 0; j < dd; j++) 
			centered[i] += ( row(j) - mean(j) ) * principal(j);
	}

	for(i = 0; i < n; i++)  var += centered[i] * centered[i];
	var /= (float)n;

    fvec r(dd);
	while( (var - varold) / var >= PRINCIPAL_ERROR && nl <= MIN_RECALC )
	{
		for(i = 0; i < dd; i++)  r(i) = 0;
		for(i = 0; i < n; i++) 
		{
			row = matrix->row(i);
			for(j = 0; j < dd; j++)  r(j) += ( row(j) - mean(j) ) * centered[i];
		}
		float mag = 0;
		for(j = 0; j < dd; j++)  mag += r(j) * r(j);
		mag = sqrt(mag);
		principal = r;
		principal /= mag;
		varold = var;
		nl++;
		for(i = 0; i < n; i++) 
		{
			row = matrix->row(i);
			for(j = 0; j < dd; j++) 
			centered[i] += ( row(j) - mean(j) ) * principal(j);
		}
		var = 0;
		for(i = 0; i < n; i++) var += centered[i] * centered[i];
		var /= (float)n;
	}	
}	


void
knnTree::sort(float *a, int *b, int l, int r)
{
  static float m;
  static int j;
  int i;

  if(r > l) {
    m = a[r]; i = l-1; j = r;
    for(;;) {
      while(a[++i] < m) if(i >= j) break;
      while(a[--j] > m) if(i >= j) break;
      if(i >= j) break;
      std::swap(a[i], a[j]);
      std::swap(b[i], b[j]);
    }
    std::swap(a[i], a[r]);
    std::swap(b[i], b[r]);
    sort(a, b, l, i-1);
    sort(a, b, i+1, r);
  }
}



node *
knnTree::buildTree(
		FeatMatrix	*pa,	
		int			n,			// number of points
		int			dd,			// dimension
		int			cn,  		// child nodes
		int			*index)
{
	if(n <= cn*cn/2)  return new node(n, index, NULL, NULL, NULL, NULL);
	int i, j;

	fmatrix *cur;
	cur = new fmatrix(n, dd);

    fvec tmp(dd);
	for(i = 0; i < n; i++)
	{
	    tmp = pa->row(index[i]);
        cur->row(i, tmp);
    }
    
	fvec principal(dd);
	getPrincipal(cur, n, dd, principal);
	fvec project(n);
	project.setval(0);
	for(i=0; i<n; i++)
	{
		tmp = cur->row(i);	
		for(j = 0; j < dd; j++)
		{
			//cout << principal(j) << " " << tmp(j) << endl;
			project(i) += principal(j) * tmp(j);
		}
	}
	
	fvec min(cn);
	fvec max(cn);
	node **children;
	children = new (node *)[cn];
	
	
	//for(i=0;i<n;i++) cout << project(i) << endl;
	//cout << endl;
	
	sort(project.getData(), index, 0, n-1);
	
	//for(i=0;i<n;i++) cout << project(i) << endl;
	//cout << endl;	
	
	int sub = n/cn;
	int actual = sub+1;
	int extra = n%cn;  
	for(j = 0, i = 0; j < cn; j++, i += actual)
	{
		if(extra-- == 0)  actual = sub;
		min(j) = project(i);
		//if(j==cn-1) cout << i+actual-1 << " " << n << " " << endl;
		max(j) = project( i + actual - 1);
		children[j] = buildTree(pa, actual, dd, cn, index+i);
    }
    return new node(cn, NULL, children, principal, min, max);
} 



knnTree::knnTree(			
		FeatMatrix	*pa,	
		int			n,			// number of points
		int			dd,			// dimension
		int			cn)  		// child nodes
{
    dim = dd;
    num = n;
    cc = cn;
	values = pa;
	index = new int[num];
	for(int i=0; i<num; i++) index[i]=i;
}

void
knnTree::makeTree()
{
  	root = buildTree(values, num, dim, cc, index);
}


void
knnTree::partialDist(
    node *terminal,
    fvec vec2)
{
    int num, i, n = terminal->cc;
    float diff, sum;
    fvec vec1(dim);
    for(num = 0; num < n; num++)
    {
        vec1 = values->row(terminal->index[num]);
        sum    = 0.0;
        for(i=0; i < dim && (sum < dist[knum-1] || dist[knum-1] == EMPTY); i++)
        {
            diff = vec1(i) - vec2(i);
            sum += diff*diff;
        }
        if((sum < dist[knum-1] || dist[knum-1] == EMPTY) && i == dim) 
        {
           for(i = knum - 1; i > 0 && nns[i-1] == EMPTY; i--);
           for(; i > 0 && sum < dist[i-1]; i--)
           {
                nns[i]  = nns[i-1];
           		dist[i] = dist[i-1];
           }
           nns[i]  = terminal->index[num];
           dist[i] = sum;
        }
    }
    return;
}



void
knnTree::knnSearch(
    node *n,
	fvec q,
	fvec b,
	int dlb)
{

    //cout << q.size() << " " << q(0)<< " " << q(1)<< " " << q(2)<< " " << q(3) << endl;
	if(n->children == NULL)
	{
	    partialDist(n, q);
	    return;
	}

	float proj = 0;
	int i, iu = n->cc -1, il = 0;
	bool lowerDone = false;
	bool upperDone = false;
	
	for(i = 0; i < dim; i++)  proj += b(i) * n->principal(i);
	if(proj <= n->max(0))
	{
		iu = 0;
		lowerDone = true;
	}
	else if( proj >= n->min(n->cc - 1) )
	{
		iu = n->cc-1;
		il = n->cc-2;
		upperDone = true;
	}
	else
	{
		il = 0;
		iu = n->cc-1;
		while(iu-il > 1)
		{
			i = (int)floor( (double)(iu + il) / 2.0 );
			if(proj <= n->min(i+1)) iu = i;
			else il = i;
		}
	}
	knnSearch(n->children[iu], q, b, dlb);
	iu++;
	float dl = 0, du = 0;
	if(lowerDone == false)  dl = proj - n->max(il);
	if(upperDone == false)  du = n->min(iu) - proj;
	while(upperDone == false || lowerDone == false)
	{
		if( (upperDone == true || dl < du) && lowerDone == false )
		{
		    float dlbl;
			dlbl=dlb + (dl*dl);
			if(dist[knum-1] < dlbl && dist[knum-1] != EMPTY)  lowerDone = true;
			else
			{
			    fvec bl(b);
			    if(n->children[il]->children != NULL)
			    {
	    			fvec tmp(n->principal);
			    	tmp *= dl;
			    	bl -= tmp;
			    }
			    knnSearch(n->children[il], q, bl, dlbl);
			    il--;
			    if(il == -1)  lowerDone = true;
			    else dl = proj - n->max(il);
			}
		}
		else
		{
		    float dlbu;
		    dlbu = dlb + (du*du);
		    if(dist[knum-1] < dlbu && dist[knum-1] != EMPTY) upperDone = true;
		    else
		    {
		    	fvec bu(b);
			    if(n->children[iu]->children != NULL)
			    {
				    fvec tmp(n->principal);
			    	tmp *= du;
			    	bu += tmp;
			    }
			    knnSearch(n->children[iu], q, bu, dlbu);
			    iu++;
			    if(iu == n->cc)  upperDone = true;
			    else du = n->min(iu) - proj;
			}
		}
	}
}



void
knnTree::findKnn(

    fvec    q,              // query point
	int		k,
    int		*mins,			// nns to return
 	float	*dists)			// distances
{
    knum   = k;
    nns    = mins;
    dist   = dists;
    fvec query(q);
    for(int i = 0; i < knum; i++)
    {
        nns[i]  = EMPTY;
        dist[i] = EMPTY;
    }   

    fvec b(query);
    knnSearch(root, query, b, 0);
}

void
knnTree::dump(node *n)
{
    int i;
    if(n == NULL) n=root;
	if(n->children==NULL) 
	{
	    cout << "leaf" << endl;
	    int *j=n->index;
	    for(int i=0; i<n->cc; i++) cout <<j[i]<<" "<<values->row(j[i])(0)<<" "<<values->row(j[i])(1)<<" "<<values->row(j[i])(2)<<" "<<values->row(j[i])(3)<<endl;
	    cout <<endl;
    }
	else
	{
	    cout << "N="<< n->cc<< " Principal Axis: ";
	    for(i=0; i<dim; i++) cout << n->principal(i)<<" ";
	    cout <<endl;
	    for(int i=0; i<cc; i++)
			cout << n->min(i) << " " << n->max(i) << endl;
		cout <<endl<<endl;
		
		for(int i=0; i<cc; i++)
			dump(n->children[i]);
		
	}
	cout << endl;
	return;
}



void
knnTree::writeNode(node* n, ofstream &tos)
{
  int i;
  if(n->children == NULL) 
  {
     tos << "leaf" << endl;
     tos << n->cc << endl;
     tos << n->index - index << endl;
     return;
  }
  tos << "node" << endl;
  tos << n->cc << endl;
  for(i = 0; i < dim; i++) tos << n->principal(i) << "\t";
  tos << endl;
  for(i = 0; i < n->cc; i++) tos << n->min(i) << "\t" << n->max(i) << endl; 
  tos << endl;
  for(i = 0; i< n->cc; i++) writeNode(n->children[i], tos);
  tos << endl;
}



void
knnTree::write(string name)
{
  int i;
  ofstream tos(name.c_str());
  tos << dim << endl << endl;
  tos << num << endl << endl;
  tos << cc << endl << endl;
  for(i = 0; i < num; i++) tos << index[i] << endl;
  tos << endl;
  writeNode(root, tos);
}


node *
knnTree::readNode(ifstream &tis)
{
  int i;
  int num; 
  string str;
  tis >>  str;
  if(str == "leaf") 
  {
    int indxOff;
    tis >> num;
    tis >> indxOff;
    return new node(num, index+indxOff, NULL, NULL, NULL, NULL);
  }
  else if(str != "node")
  {
    cerr << "knnTree::readNode '" << str << "' should be 'node'" << endl;
  }
  tis >> num;
  fvec principal(dim);
  for(i = 0; i<dim; i++) tis >> principal(i);
  fvec min(num), max(num); 
  for(i = 0; i<num; i++)
  {
    tis >> min(i);
    tis >> max(i);
  }
 
  node **children;
  children = new (node *)[num];   
  for(i = 0; i < num; i++)  children[i] = readNode(tis);
  return new node(num, NULL, children, principal, min, max);
}


void
knnTree::read(string name)
{
  int i;
  ifstream tis(name.c_str());
  string str;
  int d;
  tis >> d;
  if(d != dim) cerr << "knnTree::read dim" << d << "differs from "<<dim<<endl;
  int n;
  tis >> n;
  if(n != num) cerr << "knnTree::read num" << n << "differs from "<<num<<endl;
  int c;
  tis >> c;
  if(c != cc) cerr << "knnTree::read cc" << c << "differs from "<<cc<<endl;
  index = new int[num];
  for(i=0; i<num; i++) tis >> index[i];

  tis >> str;
  if(str != "node")
  {
     cerr << "knnTree::read(" << name << "): no root" << endl;
     exit(1);
  } 

  tis >> n;
  fvec principal(dim);
  for(i = 0; i<dim; i++) tis >> principal(i);
  fvec min(n), max(n);
  for(i = 0; i<n; i++)
  {
    tis >> min(i);
    tis >> max(i);
  }
   
  node **children;
  children = new (node *)[n];
  for(i = 0; i<n; i++) 
    children[i] = readNode(tis);
  root = new node(n, NULL, children, principal, min, max); 
}








