#include "MSignal.h"
#include "Filter.h"
#include "AutoCorrelationFFT.h"
#include "HalfWaveRectifier.h"
#include "fvec.h"
#include "Util.h"
#include "UpSampler.h"
#include "LPCResyn.h"
#include "PeakEnhancer.h"
#include "NormRMS.h"
#include "MultiPitch.h"
#include "FeatExtractor.h"

#ifndef WIN32
#include "Audio.h"
#else
#include "system/AudioDummy.h"
#endif

void multipitch(string filename)
{
  MSignal *src;
  Util util;
  src = util.sfopen(filename, MRS_SF_READ);
  src->initWindow(512, 512, 0, 0);
  MultiPitch multipitch(src->winSize());
  FeatExtractor multipitchExtractor(src, &multipitch);
  multipitchExtractor.extract(0, src->iterations());
}



void
multipitch_old(string filename)
{
  fvec num_low(3), denom_low(3); 
  fvec num_high(3), denom_high(3);
 
  //coefs are for butter(2,1000) and butter(2,1000,'high') 
  num_low(0)=0.1207; num_low(1)=0.2415; num_low(2)=0.1207;
  denom_low(0)=1.0; denom_low(1)=-0.8058; denom_low(2)=0.2888;

  num_high(0)=0.5236; num_high(1)=-1.0473; num_high(2)=0.5236;
  denom_high(0)=1.0; denom_high(1)=-0.8058; denom_high(2)=0.2888;

  Filter lowpass1(num_low, denom_low, "lpf1");
  Filter lowpass2(num_low, denom_low, "lpf2");
  Filter highpass(num_high, denom_high, "hpf");

  unsigned int i;
  MSignal *src;
  Util util;
  src = util.sfopen(filename, MRS_SF_READ);
  
  //I previously had src->initWindow(1024,1024,0,0) but LPCResyn uses 512
  src->initWindow(512, 512, 0, 0);
  //Audio audio(src->srate(), 1, AUDIO_PLAY, src->winSize());
  
  fvec window(src->winSize());
  fvec sum(src->winSize());
  fvec lowp(src->winSize());
  fvec highp(src->winSize());
  fvec maxpitches(src->iterations());
 
  int order=20;  //can adjust this
  LPCResyn lpcr(src->winSize(), src->hopSize(),order+1);
  
  HalfWaveRectifier hwr1(src->winSize()); 
  HalfWaveRectifier hwr2(src->winSize());
  HalfWaveRectifier hwr3(src->winSize());
  AutoCorrelationFFT autocorfft(src->winSize());
  UpSampler up(src->winSize(), 2); //factor=2
  NormRMS norm(src->winSize());
  /* PeriodicRemoval mr2(src->winSize(), 2);
  PeriodicRemoval mr3(src->winSize(), 3);
  PeriodicRemoval mr4(src->winSize(), 4);
  */
/*
 cerr << "iterations = " << src->iterations() << endl;
 fvec impulse;
 fvec responce;
 impulse.create(1024);
 responce.create(1024);
 impulse(0) = 1.0;
 lowpass.process(impulse, responce);
 plotter.plot_wait(responce); 
 cout << responce << endl;
 return; 
*/
  PeakEnhancer mr2(src->winSize(), 2);
  PeakEnhancer mr3(src->winSize(), 3);
  PeakEnhancer mr4(src->winSize(), 4);
  

  for (i=0; i < src->iterations(); i++)
    {
      src->get(window, i);
      lowpass1.process(window, lowp);
      highpass.process(window, highp);
      hwr1.process(highp, highp);
      lowpass2.process(highp, highp);
      hwr2.process(lowp, lowp);
      
      autocorfft.process(lowp, lowp); 
      autocorfft.process(highp, highp); 
      sum=lowp+highp;
      
      // plotter.plot_wait(lowp);
      // plotter.plot_wait(highp);

      mr2.process(sum, sum);
      mr3.process(sum, sum);
      mr4.process(sum, sum);
      hwr2.process(sum, sum);
      
      
      float max[3]; max[0]=max[1]=max[2]=0;
      int jmax[3]; jmax[0]=jmax[1]=jmax[2]=0;
      int j;
      sum.abs();
      for (j=0; j<512; j++)
	{ //max[2] is biggest, max[0] is 3rd biggest
	  if(sum(j) > max[2])
	    { max[0]=max[1]; jmax[0]=jmax[1];
	    max[1]=max[2]; jmax[1]=jmax[2];
	    max[2]=sum(j); jmax[2]=j;
	    }
	  if(sum(j) > max[1] && sum(j) < max[2])
	    { max[0]=max[1]; jmax[0]=jmax[1];
	    max[1]=sum(j); jmax[1]=j;
	 }           
	  if(sum(j) > max[0] && sum(j) < max[1])
	    { max[0]=sum(j); jmax[0]=j;
	    }
	}
      cout << 22050.0/jmax[0] <<" " <<  22050.0/jmax[1] << " " << 22050.0/jmax[2] <<endl;
      maxpitches(i)=22050.0/max[2];
    }
  
}

int 
main(int argc, char **argv)
{
  if (argc == 1)
    {
      cerr << "Usage :" << argv[0] << " filenames\n";
      exit(1);
    }
  int i;
  
  for (i=1; i < argc; i++)
    {
      multipitch_old(argv[i]);
    }
  return 0;
}


