/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* sfplay:
   Command line utility for playing soundfiles. 
*/



#include <stdio.h>
#include "FeatMatrix.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"


#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "MagDB.h"
#include "PlotExtractor.h"

#ifndef WIN32
#include "Plotter.h"
#else
#include "system/DummyPlotter.h"
#endif 

void 
plot(string filename, unsigned int lineSize)
{
  MSignal *src;
  Util util;
  src = util.sfopen(filename, MRS_SF_READ);
  src->initWindow(lineSize, lineSize, 0, 0);
  PlotExtractor pextractor(src, src->winSize());
  fvec res(src->iterations());
  pextractor.extract(0, src->iterations(), res);
  Plotter plotter;
  plotter.plot_wait(res);
}
	
int 
main(int argc, char **argv)
{
  unsigned int lineSize;
  if ((argc < 2)||(argc > 3))
    {
      cerr << "Usage :" << argv[0] << " filename [lineSize]\n";
      exit(1);
    }
  if (argc == 3) 
    lineSize = atoi(argv[2]);
  else 
    lineSize = DEFAULT_WIN_SIZE;
  cerr << "Plotting " << argv[1] << " with lineSize = " << lineSize << endl;
  plot(argv[1], lineSize);
  return 0;
}
