/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>

#include "FeatMatrix.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "defs.h"
#include "Collection.h"


/* to create the pcaWeights matrix do the following steps:
 * run covariance (some list of files you are interested) res (covariance matrix) 
 * then using some external tool (like octave) calculate 
 * the eigenvectors matrix 
 * in octave: load res, [V, D] = eig(m), PR = V' save eigen
 * Edit eigen deleting everything except PR and making a MARSYAS fmatrix 
 * This matrix is the one using for projecting the feature with pca. 
 */ 

string 
project(string srcStr, string projectStr)
{
  FeatMatrix src;
  src.read(srcStr);
  
  cout << "Projection matrix for multiplying " << srcStr << " = " << projectStr << endl;
  fmatrix projectMatrix;
  projectMatrix.read(projectStr);
 
  src *= projectMatrix;
  unsigned int c;
  vector<string> names;
  for (c =0; c < src.cols(); c++)
    {
      names.push_back("Projected");
    }
  // src.changefeatNames(names);
  string resStr = srcStr + ".prj";
  cout << "Writing " << resStr << endl;
  ofstream to(resStr.c_str());
  to << src;
  return resStr;
}

 

int
main(int argc, char **argv)
{
  if (argc == 1)
    {
      cerr << "Usage :" << argv[0] <<" collections matrix \n";
      exit(1);
    }
  unsigned int numClasses = argc -2;
  unsigned int size;
  unsigned int cl;
  unsigned int m;
  
  
  for (cl = 0; cl < numClasses; cl++)
    {
      cout << "Class " << cl << " collection is :" << argv[cl+1] << endl;
    }
  
  vector <FeatMatrix *> allMatrices;
  allMatrices.reserve(1000);

  string projectStr = argv[argc-1];
  fmatrix projectMatrix;
  projectMatrix.read(projectStr);
  
  unsigned int count = 0;
  for (cl = 0; cl < numClasses; cl++)
    {
      string collectionstr = MARSYAS_MFDIR;
      collectionstr += "/featurefiles/";
      collectionstr += argv[cl+1];
      ifstream from(collectionstr.c_str());

      string destStr = MARSYAS_MFDIR;
      destStr += "/featurefiles/";
      destStr += argv[cl+1];
      destStr += ".prj";
      ofstream to(destStr.c_str());
      
      allMatrices.push_back(new FeatMatrix());
      while (from >> *(allMatrices[count]))
	{
	  allMatrices.push_back(new FeatMatrix());
	  cout << "reading " << allMatrices[count]->name() << endl;
	  *(allMatrices[count]) *= projectMatrix;
	  to << *(allMatrices[count]) << endl;
	  count++;
	}
      allMatrices.pop_back();
      cout << "Wrote " << destStr << endl;
    }
  return 0;
}
