/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* similar:
   Command line utility for finding content-based similarity 
retrieval.
*/

#if HAVE_CONFIG_H
#include <config.h>
#endif


#include <stdio.h>
#include "FeatMatrix.h"
#include "MSignal.h"
#include "fmatrix.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"
#include "Gain.h"
#include "Hamming.h"
#include "MagFFT.h"
#include "FeatExtractor.h"
#include "Series.h"
#include "MagDB.h"
#include "Sine.h"
#include "Spectral.h"
#include "SpectralSegm.h"
#include "SfxExtractor.h"
#include "MemMFCC.h"
#include "LPC.h"
#include "MultiPitchFeature.h"
#include "DWTC.h"
#include "BeatExtractor.h"
#include "SvFeatExtractor.h"
#include "MultiPitchExtractor.h"
#include "GenreExtractor.h"
#include "ExtractorDictionary.h"
#include "FileName.h"
#include "KNN.h"
#include "Gaussian.h"
#include "Classifier.h"


#include <map>
#include <string.h>
using namespace std;

#include "MSignal.h"
#include "SoundFileIO.h"
#include "defs.h"
#include "Util.h"

#ifndef WIN32
#include "Audio.h"
#else
#include "system/AudioDummy.h"
#endif


void 
play(float seconds, string filename, MSignal *dest)
{
  /* string commandStr = "xmms ";
  commandStr += filename;
  system(commandStr.c_str());
  return;
  */

  MSignal *src;
  Util util;
  src = util.sfopen(filename, MRS_SF_READ);
  if (src == NULL) 
    exit(1);
  src->initWindow(1024, 1024, 0, 0);
  fvec window(src->winSize());

  float iterations = (seconds * src->srate()) / src->winSize();
  if ((iterations >= src->iterations())||(seconds == -1.0))
    iterations = src->iterations();
  for (unsigned int i=0; i< iterations; i++)
    {
      src->get(window,i);
      dest->put(window,i);
      // dest->put(window,i);
    }
  delete src;
}


void similar(string extractorStr, string audiofileStr, int k, 
	     string collectionStr)
{
  unsigned int i;
  /* Prepare name (remove full path) for feature matrix */ 
  FileName fromFile(audiofileStr);
  string fromName = fromFile.name();  
  string mf("mf");

  

  FeatMatrix* qfeatMatrix = new FeatMatrix();
  
  if (fromFile.ext() == mf)		// File is a feature matrix
    {
      cout << "reading from " << audiofileStr << endl;
      qfeatMatrix->read(audiofileStr);
    }
  else					// Audio file extract features
    {
      MSignal *src;
      Util util;
      src = util.sfopen(audiofileStr, MRS_SF_READ);
      src->initWindow(512, 512, 0, 0);
      
      /* Special window sizes for specific extractors */ 
      if (extractorStr == "LPC")
	src->initWindow(512, 128, 0, 0);
      if (extractorStr == "BEAT") 
	src->initWindow(65536, 65536/2,0,0);
      if (extractorStr == "DWTC")
	src->initWindow(16384, 512, 0, 0);
      if (extractorStr == "MPITCH")
	src->initWindow(512, 512, 0, 0);
      
      /* Extractor dictionary contains all the available 
	 extractors for feature calculation */ 
      ExtractorDictionary dict(src);
      
      Extractor* extr = dict.getExtractor(extractorStr);
      
      
      /* find appropriate extractor and feature matrix result */ 
      delete qfeatMatrix;
      unsigned int rows = dict.getRows(extractorStr);
      unsigned int cols = dict.getCols(extractorStr);
      /* query feature matrix */
      qfeatMatrix = new FeatMatrix(rows, cols);
      
      cout << "Extracting features " << endl;
      extr->extract(*qfeatMatrix);
      qfeatMatrix->setName(fromName);
    }
  
  
      
  /* Read the training collection */
  FeatMatrix trainMatrix;
  trainMatrix.read(collectionStr);
  
  /* 
  vector <FeatMatrix *> allMatrices;
  unsigned int count = 0;
  allMatrices.push_back(new FeatMatrix());
  ifstream from(collectionStr.c_str());
  while (from >> *(allMatrices[count]))
    {
      allMatrices.push_back(new FeatMatrix());
      count++;
    }

  
  allMatrices.pop_back();
  vector<string> labels;
  labels.reserve(100);

  // Label each individual feature matrix as a separate class 
  for (i=0; i < count; i++)
    {
      labels.push_back(allMatrices[i]->name());
    }
  
  for (i=0; i < count; i++)
    {
      // allMatrices[i]->label(labels, i);
    }
  
  FeatMatrix trainMatrix(allMatrices);
  */ 


  KNN classifier(1);
  // Gaussian classifier;
  
  classifier.train(&trainMatrix,1);

  // Return the rank list 
  vector<string> rlist;
  cout << "similar rows " << (*qfeatMatrix).rows() << endl;
  
  rlist = classifier.rankList(*qfeatMatrix, k);
  cout << endl;
  cout << "The " << k << " most similar files are:" << endl;
  
  AuFileIO *dest1;
  dest1 = new AuFileIO("similar.au", MRS_SF_WRITE);
  dest1->initMSignal(22050, 1);
  dest1->init();
  dest1->initWindow(1024, 1024, 0, 0);


  AuFileIO *dest2;
  dest2 = new AuFileIO("random.au", MRS_SF_WRITE);
  dest2->initMSignal(22050, 1);
  dest2->init();
  dest2->initWindow(1024, 1024, 0, 0);

  // Audio audio(22050,1, MRS_AUDIO_PLAY, 1024);
  
  // play(10, audiofileStr, &audio);
  
  string commandStr = "mpg123 ";
  commandStr += audiofileStr;
  commandStr += " ";
  
  // for (i=0; i < rlist.size(); i++)
  for (i=1; i < 5; i++)
    {
      // cout << i << ". " << rlist[i] << endl;
      // play(10, rlist[i], &audio);
      commandStr += rlist[i];
      commandStr += " ";
    }
  system(commandStr.c_str());
  /* 
  unsigned int rind;
  srand(5);
  for (i=0; i < 3; i++)
    {
      rind = (unsigned int)(((float)rand() / (float)(RAND_MAX)) * rlist.size());
      cout << "rind = " << rind << endl;
      cout << i << ". " << rlist[rind] << endl;
      if (i==0) 
	play(10, rlist[i], dest2);
      else
	play(15, rlist[rind], dest2);
    }
  */
      
  

}



	
int 
main(int argc, char **argv)
{ 
  cout << "New similar" << endl;
  if (argc != 2)
    {
      cerr << "Usage :" << argv[0] << " audiofile " << endl;
      exit(1);
    }
  // string extractorStr = argv[1];
  string extractorStr = "GENRE";
  string audiofileStr = argv[1];
  //  string audiofileStr = argv[2];
  // int k = atoi(argv[3]);
  int k = 10;
  // string cStr = argv[4];
  string cStr = "mp3lib1";
  
  string collectionStr = MARSYAS_MFDIR;
  // cStr = cStr.substr(0, cStr.size()-3); // remove .mf
  collectionStr += "/featurefiles/";
  collectionStr += cStr;
  collectionStr += extractorStr;
  // collectionStr += ".mff";
  
  cout << "\n Finding the " << k << " most similar files to " << audiofileStr << " in collection: "  << endl;
  cout << collectionStr << endl << endl;
  similar(extractorStr, audiofileStr, k, collectionStr);
  return 0;
}
